@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -30,11 +30,11 @@
 (**************************************************************************)
 
 
-exception NotImplemented of string
 exception Unsupported of string
 
+val jessie_emitter: Emitter.emitter
+
 val fatal          : ('a,Format.formatter,unit,'b) format4 -> 'a
-val notimplemented : ('a,Format.formatter,unit,'b) format4 -> 'a
 val unsupported    : ('a,Format.formatter,unit,'b) format4 -> 'a
 val warning        : ('a,Format.formatter,unit) format -> 'a
 val warn_general   : ('a,Format.formatter,unit) format -> 'a
@@ -84,12 +84,17 @@ val check_types : Cil_types.file -> unit
 val integral_type_size_in_bits : Cil_types.typ -> int
 
 val max_value_of_integral_type :
-  ?bitsize:int -> Cil_types.typ -> My_bigint.t
+  ?bitsize:int -> Cil_types.typ -> Integer.t
 
 val min_value_of_integral_type :
-  ?bitsize:int -> Cil_types.typ -> My_bigint.t
+  ?bitsize:int -> Cil_types.typ -> Integer.t
 
-val all_integral_types : (string, Cil_types.typ * int) Hashtbl.t
+(* iter over existing integral types in alphabetical order. *)
+val iter_integral_types: (string -> Cil_types.typ -> int -> unit) -> unit
+
+(* fold over existing integral types in alphabetical order. *)
+val fold_integral_types:
+  (string -> Cil_types.typ -> int -> 'a -> 'a) -> 'a -> 'a
 
 val term_of_var : Cil_types.varinfo -> Cil_types.term
 
@@ -217,4 +222,28 @@ val visit_and_push_statements :
 
 val print_to_stdout : Cil_types.file -> unit
 
-val constant_expr : ?loc:Cil_datatype.Location.t -> My_bigint.t -> Cil_types.exp
+val constant_expr : ?loc:Cil_datatype.Location.t -> Integer.t -> Cil_types.exp
+
+
+open Cil_types
+
+type opaque_term_env = {
+  term_lhosts: term_lhost Cil_datatype.Varinfo.Map.t;
+  terms: term Cil_datatype.Varinfo.Map.t;
+  vars: logic_var Cil_datatype.Varinfo.Map.t;
+}
+
+type opaque_exp_env = { exps: exp Cil_datatype.Varinfo.Map.t }
+
+val force_term_to_exp: term -> exp * opaque_term_env
+val force_back_exp_to_term: opaque_term_env -> exp -> term
+val force_exp_to_term: exp -> term
+val force_lval_to_term_lval: lval -> term_lval
+val force_term_offset_to_offset: term_offset -> offset * opaque_term_env
+val force_back_offset_to_term_offset: opaque_term_env -> offset -> term_offset
+val force_exp_to_predicate: exp -> predicate named
+val force_exp_to_assertion: exp -> code_annotation
+val force_term_lval_to_lval: term_lval -> lval * opaque_term_env
+val force_back_lval_to_term_lval: opaque_term_env -> lval -> term_lval
+
+val predicate: location -> predicate -> predicate named
