@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -29,20 +29,20 @@
 (*                                                                        *)
 (**************************************************************************)
 
-open Plugin
 include Plugin.S
 
-module ProjectName: Plugin.String
-module Behavior: Plugin.String
-module Analysis: Plugin.Bool
-module WhyOpt: Plugin.String_set
-module JcOpt: Plugin.String_set
-module GenOnly: Plugin.Bool
-module InferAnnot: Plugin.String
-module AbsDomain: Plugin.String
-module Atp: Plugin.String
-module CpuLimit: Plugin.Int
-module HintLevel: Plugin.Int
+module ProjectName: Parameter_sig.String
+module Behavior: Parameter_sig.String
+module Analysis: Parameter_sig.Bool
+module WhyOpt: Parameter_sig.String_set
+module Why3Opt: Parameter_sig.String_set
+module JcOpt: Parameter_sig.String_set
+module GenOnly: Parameter_sig.Bool
+module InferAnnot: Parameter_sig.String
+module AbsDomain: Parameter_sig.String
+module Atp: Parameter_sig.String
+module CpuLimit: Parameter_sig.Int
+module HintLevel: Parameter_sig.Int
 
 (*
 Local Variables:
