@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -46,11 +46,10 @@
 
 *)
 
+
 (* Import from Cil *)
 open Cil_types
 open Cil
-open Cilutil
-open Cil_datatype
 open Ast_info
 open Extlib
 
@@ -58,7 +57,8 @@ open Visitor
 
 (* Utility functions *)
 open Common
-open Integer
+
+let label_here = LogicLabel(None,"Here")
 
 (*****************************************************************************)
 (* Retype variables of array type.                                           *)
@@ -196,10 +196,9 @@ class retypeArrayVariables =
   in
 object(self)
 
-  inherit Visitor.generic_frama_c_visitor
-    (Project.current ()) (Cil.inplace_visit ()) as super
+  inherit Visitor.frama_c_inplace
 
-  method vvdec v =
+  method! vvdec v =
     if isArrayType v.vtype && not (Cil_datatype.Varinfo.Set.mem v !varset) then
       begin
         assert (not v.vformal);
@@ -209,7 +208,7 @@ object(self)
         varset := Cil_datatype.Varinfo.Set.add v !varset;
         (* Change the variable type *)
         let newty =
-          if My_bigint.gt (array_size v.vtype) My_bigint.zero then
+          if Integer.gt (array_size v.vtype) Integer.zero then
             begin
               (* Change the type into "reference" type, that behaves almost like
                * a pointer, except validity is ensured.
@@ -231,7 +230,7 @@ object(self)
       end;
     DoChildren
 
-  method vlogic_var_decl lv =
+  method! vlogic_var_decl lv =
     if not (Cil_datatype.Logic_var.Hashtbl.mem lvar_to_strawlvar lv) &&
       app_term_type isArrayType false lv.lv_type then
       begin
@@ -240,9 +239,9 @@ object(self)
         let elemty = force_app_term_type element_type lv.lv_type in
         lvarset := Cil_datatype.Logic_var.Set.add lv !lvarset;
         let newty =
-          if My_bigint.gt 
+          if Integer.gt 
             (force_app_term_type array_size lv.lv_type) 
-            My_bigint.zero 
+            Integer.zero 
           then
             begin
               let size =
@@ -261,10 +260,10 @@ object(self)
       end;
       DoChildren
 
-  method vglob_aux g = match g with
+  method! vglob_aux g = match g with
     | GVar(v,_init,_loc) ->
         (* Make sure variable declaration is treated before definition *)
-        ignore (visitFramacVarDecl (self:>frama_c_visitor) v);
+        ignore (Cil.visitCilVarDecl (self:>Cil.cilVisitor) v);
         if Cil_datatype.Varinfo.Set.mem v !allocvarset then
           (* Allocate memory for new reference variable *)
           let ty = Cil_datatype.Varinfo.Hashtbl.find var_to_array_type v in
@@ -277,15 +276,17 @@ object(self)
 *)
           (* Define a global validity invariant *)
           let p =
-            Pvalid_range(
-              variable_term v.vdecl (cvar_to_lvar v),
-              constant_term v.vdecl My_bigint.zero,
-              constant_term v.vdecl (My_bigint.pred size))
+            Logic_const.pvalid_range
+              ~loc:v.vdecl
+              (label_here,
+               variable_term v.vdecl (cvar_to_lvar v),
+               constant_term v.vdecl Integer.zero,
+               constant_term v.vdecl (Integer.pred size))
           in
           let globinv =
 	    Cil_const.make_logic_info (unique_logic_name ("valid_" ^ v.vname)) in
-          globinv.l_labels <- [ LogicLabel(None,"Here") ];
-          globinv.l_body <- LBpred (predicate v.vdecl p);
+          globinv.l_labels <- [ label_here ];
+          globinv.l_body <- LBpred p;
           attach_globaction (fun () -> Logic_utils.add_logic_function globinv);
           ChangeTo [g;GAnnot(Dinvariant (globinv,v.vdecl),v.vdecl)]
         else DoChildren
@@ -293,17 +294,17 @@ object(self)
     | GCompTag _ | GType _ | GCompTagDecl _ | GEnumTagDecl _
     | GEnumTag _ | GAsm _ | GPragma _ | GText _ -> SkipChildren
 
-  method vfunc f =
+  method! vfunc f =
     (* First change type of local array variables *)
-    List.iter (ignore $ visitFramacVarDecl (self:>frama_c_visitor)) f.slocals;
-    List.iter (ignore $ visitFramacVarDecl (self:>frama_c_visitor)) f.sformals;
+    List.iter (ignore $ Cil.visitCilVarDecl (self:>Cil.cilVisitor)) f.slocals;
+    List.iter (ignore $ Cil.visitCilVarDecl (self:>Cil.cilVisitor)) f.sformals;
     (* Then allocate/deallocate memory for those that need it *)
     List.iter (fun v ->
       if Cil_datatype.Varinfo.Set.mem v !allocvarset then
         let ty = Cil_datatype.Varinfo.Hashtbl.find var_to_array_type v in
         let elemty = element_type ty in
         let ast = mkalloc_array_statement v elemty 
-          (My_bigint.to_int64 (array_size ty)) v.vdecl 
+          (Integer.to_int64 (array_size ty)) v.vdecl 
         in
         add_pending_statement ~beginning:true ast;
         let fst = mkfree_statement v v.vdecl in
@@ -311,16 +312,16 @@ object(self)
     ) f.slocals;
     DoChildren
 
-  method vlval lv =
+  method! vlval lv =
     ChangeDoChildrenPost (preaction_lval lv, postaction_lval)
 
-  method vterm_lval =
-    do_on_term_lval (Some preaction_lval, Some postaction_lval)
+  method! vterm_lval lv =
+    do_on_term_lval (Some preaction_lval, Some postaction_lval) lv
 
-  method vexpr e =
+  method! vexpr e =
     ChangeDoChildrenPost (preaction_expr e, fun x -> x)
 
-  method vterm =
+  method! vterm =
     do_on_term (Some preaction_expr, None)
 
 end
@@ -339,6 +340,15 @@ let retype_array_variables file =
 (* Retype logic functions/predicates with structure parameters or return.    *)
 (*****************************************************************************)
 
+let model_fields_table = Hashtbl.create 7
+
+let model_fields compinfo =
+  (* Format.eprintf "looking in model_fields_table for %s@." compinfo.cname; *)
+  try
+    Hashtbl.find model_fields_table compinfo.ckey
+  with Not_found -> []
+
+
 (* logic parameter:
  * - change parameter type to pointer to structure
  * - change uses of parameters in logical annotations
@@ -403,10 +413,9 @@ class retypeLogicFunctions =
   in
 object
 
-  inherit Visitor.generic_frama_c_visitor
-    (Project.current ()) (Cil.inplace_visit ()) as super
+  inherit Visitor.frama_c_inplace
 
-  method vannotation =
+  method! vannotation =
     let return_type ty =
       change_result_type := false;
       match ty with
@@ -419,7 +428,7 @@ object
             end
         | Ctype _ | Ltype _ | Lvar _ | Linteger | Lreal | Larrow _ -> ty
     in
-    let rec annot = function
+    let annot = function
       | Dfun_or_pred (li,loc) ->
           List.iter var li.l_profile;
           begin
@@ -446,13 +455,29 @@ object
           end
       | Dtype _ | Dlemma _ | Dinvariant _ | Dvolatile _  -> DoChildren
       | Daxiomatic _ -> DoChildren (* FIXME: correct ? *)
-      | Dmodel_annot _ -> DoChildren (* FIXME: correct ? *)
+      | Dmodel_annot (mi,_) -> 
+          begin
+            match unrollType mi.mi_base_type with
+              | TComp (ci, _, _) ->
+                  begin
+                    let l = 
+                      try Hashtbl.find model_fields_table ci.ckey
+                      with Not_found -> []
+                    in
+                    (* Format.eprintf "filling model_fields_table for %s@." ci.cname; *)
+                    Hashtbl.replace model_fields_table ci.ckey (mi::l)
+                  end
+              | _ -> 
+              Common.unsupported "model field only on structures"
+          end;
+          DoChildren (* FIXME: correct ? *)
+      | Dcustom_annot _ -> DoChildren (* FIXME: correct ? *)
     in annot
 
-  method vterm_lval tlv =
+  method! vterm_lval tlv =
     ChangeDoChildrenPost (tlv, postaction_term_lval)
 
-  method vterm t =
+  method! vterm t =
     let preaction_term t =
       match t.term_node with
         | Tapp(callee,labels,args) ->
@@ -487,7 +512,7 @@ object
     in
     ChangeDoChildrenPost (preaction_term t, postaction_term)
 
-  method vpredicate = function
+  method! vpredicate = function
     | Papp(callee,labels,args) ->
         let args = List.map (fun arg ->
           (* Type of [arg] has not been changed. *)
@@ -602,12 +627,12 @@ class expandStructAssign () =
             expand_assign newlv newe (direct_element_type ty) loc
           in
           let rec all_elem acc i =
-            if My_bigint.ge i My_bigint.zero
-            then all_elem (elem i @ acc) (My_bigint.pred i) 
+            if Integer.ge i Integer.zero
+            then all_elem (elem i @ acc) (Integer.pred i) 
             else acc
           in
           assert (not (is_reference_type ty));
-          all_elem [] (My_bigint.pred (direct_array_size ty))
+          all_elem [] (Integer.pred (direct_array_size ty))
       | _ -> [Set (lv, e, loc)]
   in
 
@@ -626,19 +651,19 @@ class expandStructAssign () =
             expand newlv (direct_element_type ty) loc
           in
           let rec all_elem acc i =
-            if My_bigint.ge i My_bigint.zero then
-              all_elem (elem i @ acc) (My_bigint.pred i)
+            if Integer.ge i Integer.zero then
+              all_elem (elem i @ acc) (Integer.pred i)
             else acc
           in
           assert (not (is_reference_type ty));
-          all_elem [] (My_bigint.pred (direct_array_size ty))
+          all_elem [] (Integer.pred (direct_array_size ty))
       | _ -> [ lv ]
   in
 
 object(self)
 
-  inherit Visitor.frama_c_inplace as super
-  method vglob_aux =
+  inherit Visitor.frama_c_inplace
+  method! vglob_aux =
     let retype_func fvi =
       let formal (n,ty,a) =
         let ty = 
@@ -653,7 +678,7 @@ object(self)
       in
       let rt = 
         if isStructOrUnionType rt then
-          mkTRef rt "Norm.vgloab_aux(2)" 
+          mkTRef rt "Norm.vglob_aux(2)" 
         else rt 
       in
       fvi.vtype <- TFun(rt,params,isva,a)
@@ -667,7 +692,7 @@ object(self)
       | GVar _  | GCompTag _ | GType _ | GCompTagDecl _ | GEnumTagDecl _
       | GEnumTag _ | GAsm _ | GPragma _ | GText _ -> SkipChildren
 
-  method vfunc f =
+  method! vfunc f =
     let var v =
       if isStructOrUnionType v.vtype then
         let newv = copyVarinfo v (unique_name ("v_" ^ v.vname)) in
@@ -707,19 +732,14 @@ object(self)
     setReturnType f rt;
     DoChildren
 
-  method vbehavior b =
-    let kf = Extlib.the self#current_kf in
-    let ki = self#current_kinstr in
-    let old = Property.ip_all_of_behavior kf ki b in
+  method! vbehavior b =
     let lval loc lv = expand lv (typeOfLval lv) loc in
     let term t = match t.term_node with
       | TLval tlv ->
-          let lv,env =
-            !Db.Properties.Interp.force_term_lval_to_lval tlv
-          in
+          let lv,env = Common.force_term_lval_to_lval tlv in
           let lvlist = lval t.term_loc lv in
           let tslvlist =
-            List.map (!Db.Properties.Interp.force_back_lval_to_term_lval env)
+            List.map (Common.force_back_lval_to_term_lval env)
               lvlist
           in
           List.map (fun
@@ -732,6 +752,7 @@ object(self)
       | TStartOf _
       | TConst _
       | TCastE _
+      | TLogic_coerce _
       | Tat _
       | TAddrOf _
       | Tapp _
@@ -745,7 +766,7 @@ object(self)
       | TSizeOf _ | TSizeOfE _ | TSizeOfStr _ | TAlignOf _ | TAlignOfE _
       | Tlambda _ | TDataCons _ | Tbase_addr _ | TBinOp _ | TUnOp _
       | Tblock_length _ | TCoerce _ | TCoerceE _ | TUpdate _
-      | Ttypeof _ | Ttype _ | Tlet _ -> assert false
+      | Ttypeof _ | Ttype _ | Tlet _ | Toffset _ -> assert false
     in
     let zone idts =
       List.map Logic_const.new_identified_term (term idts.it_content)
@@ -759,15 +780,36 @@ object(self)
       in
       List.map (fun z -> z, froms) zl
     in
-    (match b.b_assigns with
-        WritesAny -> ()
-      | Writes l ->
-        b.b_assigns <- Writes (List.flatten (List.map assign l)));
-    let props = Property.ip_all_of_behavior kf ki b in
-    Property_status.merge ~old props;
-    DoChildren
+    let assigns = (* b.b_assigns *)
+      (* the following may stop completely the execution of the plugin ??? *)
+      (**)
+      (match b.b_assigns with
+          WritesAny -> WritesAny
+        | Writes l -> Writes (List.flatten (List.map assign l)))
+      (**)
+    in
+(*    Format.eprintf "[Norm.vbehavior] b_allocation = ";
+    begin
+      match b.b_allocation with
+        | FreeAllocAny ->
+            Format.eprintf "FreeAllocAny@."
+        | FreeAlloc(l1,l2) ->
+            Format.eprintf "FreeAlloc(%d,%d)@." (List.length l1) (List.length l2)
+    end;
+*)
+    let new_bhv =
+      Cil.mk_behavior
+        ~name:b.b_name
+        ~requires:b.b_requires
+        ~assumes:b.b_assumes
+        ~post_cond:b.b_post_cond
+        ~assigns:assigns
+        ~allocation:(Some b.b_allocation)
+        ()
+    in
+    ChangeDoChildrenPost(new_bhv,fun x -> x)
 
-  method vstmt_aux s = match s.skind with
+  method! vstmt_aux s = match s.skind with
     | Return(Some e,loc) ->
         (* Type of [e] has not been changed by retyping formals and return. *)
         if isStructOrUnionType (typeOf e) then
@@ -787,7 +829,7 @@ object(self)
         else SkipChildren
     | _ -> DoChildren
 
-  method vinst =
+  method! vinst =
     function
       | Set(lv,e,loc) ->
           (* Type of [e] has not been changed by retyping formals and return. *)
@@ -836,10 +878,10 @@ object(self)
       | Asm _ | Skip _ -> SkipChildren
       | Code_annot _ -> assert false
 
-  method vterm_lval tlv =
+  method! vterm_lval tlv =
     ChangeDoChildrenPost (tlv, postaction_term_lval)
 
-  method vterm t =
+  method! vterm t =
     (* Renormalize the term tree. *)
     let postaction t =
       match t.term_node with
@@ -899,22 +941,21 @@ class retypeStructVariables =
           if Cil_datatype.Logic_var.Set.mem v !lvarset then
             add_deref host v
           else
-            opt_app
+            Extlib.may_map
               (fun cv ->
                  if Cil_datatype.Varinfo.Set.mem cv !varset then
                    add_deref host v
                  else host
-              ) host v.lv_origin
+              ) ~dft:host v.lv_origin
       | TMem _ | TResult _ -> host
     in
     host, off
   in
 object(self)
 
-  inherit Visitor.generic_frama_c_visitor
-    (Project.current ()) (Cil.inplace_visit ()) as super
+  inherit Visitor.frama_c_inplace
 
-  method vvdec v =
+  method! vvdec v =
     if isStructOrUnionType v.vtype && not v.vformal then
       begin
         v.vtype <- mkTRef v.vtype "Norm.vvdec";
@@ -922,7 +963,7 @@ object(self)
       end;
     DoChildren
 
-  method vquantifiers vl =
+  method! vquantifiers vl =
     List.iter (fun v ->
                  (* Only iterate on logic variable with C type *)
                  if app_term_type (fun _ -> true) false v.lv_type then
@@ -933,7 +974,7 @@ object(self)
               ) vl;
     DoChildren
 
-  method vlogic_var_decl v =
+  method! vlogic_var_decl v =
     let newty =
       app_term_type
         (fun ty ->
@@ -943,7 +984,7 @@ object(self)
     v.lv_type <- newty;
     DoChildren
 
-  method vglob_aux = function
+  method! vglob_aux = function
     | GVar (_,_,_) as g ->
         let postaction = function
           | [GVar (v,_,_)] ->
@@ -955,16 +996,18 @@ object(self)
 *)
                 (* Define a global validity invariant *)
                 let p =
-                  Pvalid_range(
-                    variable_term v.vdecl (cvar_to_lvar v),
-                    constant_term v.vdecl My_bigint.zero,
-                    constant_term v.vdecl My_bigint.zero)
+                  Logic_const.pvalid_range
+                    ~loc:v.vdecl
+                    (label_here,
+                     variable_term v.vdecl (cvar_to_lvar v),
+                     constant_term v.vdecl Integer.zero,
+                     constant_term v.vdecl Integer.zero)
                 in
                 let globinv =
 		  Cil_const.make_logic_info (unique_logic_name ("valid_" ^ v.vname))
 		in
-                globinv.l_labels <- [ LogicLabel(None, "Here") ];
-                globinv.l_body <- LBpred (predicate v.vdecl p);
+                globinv.l_labels <- [ label_here ];
+                globinv.l_body <- LBpred p;
                 attach_globaction
 		  (fun () -> Logic_utils.add_logic_function globinv);
                 [g; GAnnot(Dinvariant (globinv,v.vdecl),v.vdecl)]
@@ -976,10 +1019,10 @@ object(self)
     | GCompTag _ | GType _ | GCompTagDecl _ | GEnumTagDecl _
     | GEnumTag _ | GAsm _ | GPragma _ | GText _ -> SkipChildren
 
-  method vfunc f =
+  method! vfunc f =
     (* First change type of local structure variables *)
-    List.iter (ignore $ visitFramacVarDecl (self:>frama_c_visitor)) f.slocals;
-    List.iter (ignore $ visitFramacVarDecl (self:>frama_c_visitor)) f.sformals;
+    List.iter (ignore $ Cil.visitCilVarDecl (self:>Cil.cilVisitor)) f.slocals;
+    List.iter (ignore $ Cil.visitCilVarDecl (self:>Cil.cilVisitor)) f.sformals;
     (* Then allocate/deallocate memory for those that need it *)
     List.iter (fun v ->
       if Cil_datatype.Varinfo.Set.mem v !varset then
@@ -992,12 +1035,12 @@ object(self)
     ) f.slocals;
     DoChildren
 
-  method vlval lv =
+  method! vlval lv =
     ChangeDoChildrenPost (lv, postaction_lval)
 
-  method vterm_lval lv = ChangeDoChildrenPost(lv, postaction_tlval)
+  method! vterm_lval lv = ChangeDoChildrenPost(lv, postaction_tlval)
 
-  method vexpr e =
+  method! vexpr e =
     (* Renormalize the expression tree. *)
     let postaction e = match e.enode with
       | AddrOf(Mem e,NoOffset) -> e
@@ -1005,7 +1048,7 @@ object(self)
     in
     ChangeDoChildrenPost (e, postaction)
 
-  method vterm t =
+  method! vterm t =
     (* Renormalize the term tree. *)
     let postaction t =
       match t.term_node with
@@ -1131,20 +1174,20 @@ class retypeAddressTaken =
           if Cil_datatype.Logic_var.Set.mem v !lvarset then
             add_deref host v.lv_type
           else
-            opt_app
+            Extlib.may_map
               (fun cv ->
                  if Cil_datatype.Varinfo.Set.mem cv !varset then
                    add_deref host (Ctype cv.vtype)
                  else host
-              ) host v.lv_origin
+              ) ~dft:host v.lv_origin
       | TResult _ | TMem _ -> host
-    in match lastTermOffset off with
+    in match Logic_const.lastTermOffset off with
       | TField (fi,_) ->
           if Cil_datatype.Fieldinfo.Set.mem fi !fieldset then
             (TMem
-               (Logic_utils.mk_dummy_term (TLval(host,off)) fi.ftype),TNoOffset)
+               (Logic_const.term (TLval(host,off)) (Ctype fi.ftype)),TNoOffset)
           else host,off
-      | TIndex _ | TNoOffset -> host,off
+      | TModel _ | TIndex _ | TNoOffset -> host,off
   in
   let postaction_expr e = match e.enode with
     | AddrOf(Var _v,NoOffset) ->
@@ -1169,10 +1212,11 @@ class retypeAddressTaken =
     | TAddrOf((TVar _ | TResult _), TNoOffset) -> assert false
     | TAddrOf(TMem t1,TNoOffset) -> t1
     | TAddrOf(_,off) ->
-        begin match lastTermOffset off with
+        begin match Logic_const.lastTermOffset off with
           | TField(fi,_) ->
               if Cil_datatype.Fieldinfo.Set.mem fi !fieldset then assert false
               else t
+          | TModel _ -> Common.unsupported "Model field"
           | TIndex _ -> t
           | TNoOffset -> assert false (*unreachable*)
         end
@@ -1182,10 +1226,9 @@ class retypeAddressTaken =
   let in_funspec = ref false in
 object
 
-  inherit Visitor.generic_frama_c_visitor
-    (Project.current ()) (Cil.inplace_visit ()) as super
+  inherit Visitor.frama_c_inplace
 
-  method vglob_aux = function
+  method! vglob_aux = function
     | GVar(v,_,_) ->
         if retypable_var v then
           begin
@@ -1209,7 +1252,7 @@ object
     | GType _ | GCompTagDecl _ | GEnumTagDecl _ | GEnumTag _
     | GAsm _ | GPragma _ | GText _ -> SkipChildren
 
-  method vfunc f =
+  method! vfunc f =
     (* Change types before code. *)
     let formals,locals,pairs =
       List.fold_right (fun v (fl,ll,pl) ->
@@ -1253,11 +1296,11 @@ object
     ) f.slocals;
     DoChildren
 
-  method vspec funspec =
+  method! vspec _ =
     in_funspec := true;
-    ChangeDoChildrenPost (funspec, fun x -> in_funspec := false; x)
+    DoChildrenPost (fun x -> in_funspec := false; x)
 
-  method vlogic_var_use v =
+  method! vlogic_var_use v =
     if !in_funspec then
       match v.lv_origin with
         | None -> SkipChildren
@@ -1272,15 +1315,15 @@ object
         DoChildren
       end
 
-  method vlogic_var_decl v = if retypable_lvar v then retype_lvar v; DoChildren
+  method! vlogic_var_decl v = if retypable_lvar v then retype_lvar v; DoChildren
 
-  method vlval lv = ChangeDoChildrenPost (lv, postaction_lval)
+  method! vlval lv = ChangeDoChildrenPost (lv, postaction_lval)
 
-  method vterm_lval lv = ChangeDoChildrenPost (lv, postaction_tlval)
+  method! vterm_lval lv = ChangeDoChildrenPost (lv, postaction_tlval)
 
-  method vexpr e = ChangeDoChildrenPost(e, postaction_expr)
+  method! vexpr e = ChangeDoChildrenPost(e, postaction_expr)
 
-  method vterm t = ChangeDoChildrenPost(t,postaction_term)
+  method! vterm t = ChangeDoChildrenPost(t,postaction_term)
 
 end
 
@@ -1388,10 +1431,9 @@ class retypeFields =
   in
 object
 
-  inherit Visitor.generic_frama_c_visitor
-    (Project.current ()) (Cil.inplace_visit ()) as super
+  inherit Visitor.frama_c_inplace
 
-  method vglob_aux = function
+  method! vglob_aux = function
     | GCompTag (compinfo,_) ->
         let field fi =
           if isStructOrUnionType fi.ftype then
@@ -1416,16 +1458,16 @@ object
     | GType _ | GCompTagDecl _ | GEnumTagDecl _
     | GEnumTag _ | GAsm _ | GPragma _ | GText _ -> SkipChildren
 
-  method vlval lv =
+  method! vlval lv =
     ChangeDoChildrenPost (lv, postaction_lval)
 
-  method vterm_lval =
+  method! vterm_lval =
     do_on_term_lval (None,Some postaction_lval)
 
-  method vexpr e =
+  method! vexpr e =
     ChangeDoChildrenPost(e, postaction_expr)
 
-  method vterm =
+  method! vterm =
     do_on_term (None,Some postaction_expr)
 
 end
@@ -1441,10 +1483,9 @@ let retype_fields file =
 class retypeTypeTags =
 object
 
-  inherit Visitor.generic_frama_c_visitor
-    (Project.current ()) (Cil.inplace_visit ()) as super
+  inherit Visitor.frama_c_inplace
 
-  method vterm t = match t.term_node with
+  method! vterm t = match t.term_node with
     | Ttype ty -> ChangeTo ({ t with term_node = Ttype(TPtr(ty,[])) })
     | _ -> DoChildren
 
@@ -1457,8 +1498,6 @@ let retype_type_tags file =
 (* Retype pointers to base types.                                            *)
 (*****************************************************************************)
 
-let debugtab = Hashtbl.create 0
-
 (* Retype pointer to base type T to pointer to struct S with:
  * - if T is [TVoid], no field in S
  * - otherwise, a single field of type T in S
@@ -1466,11 +1505,11 @@ let debugtab = Hashtbl.create 0
 class retypeBasePointer =
 
   (* Correspondance between a base type and its wrapper structure type *)
-  let type_wrappers : typ Typ.Hashtbl.t = Typ.Hashtbl.create 17 in
+  let type_wrappers = Cil_datatype.Typ.Hashtbl.create 17 in
   (* Store which types are wrapper types *)
-  let auto_type_wrappers = ref Typ.Set.empty in
+  let auto_type_wrappers = ref Cil_datatype.Typ.Set.empty in
 
-  let is_wrapper_type ty = Typ.Set.mem ty !auto_type_wrappers in
+  let is_wrapper_type ty = Cil_datatype.Typ.Set.mem ty !auto_type_wrappers in
 
   let new_wrapper_for_type_no_sharing ty =
     (* Choose name t_P for the wrapper and t_M for the field *)
@@ -1496,15 +1535,16 @@ object(self)
      *)
     let ty = typeRemoveAttributes ["const";"volatile"] (unrollType ty) in
     try
-      Typ.Hashtbl.find type_wrappers ty
+      Cil_datatype.Typ.Hashtbl.find type_wrappers ty
     with Not_found ->
       (* Construct a new wrapper for this type *)
       let wrapper_def,wrapper_type = new_wrapper_for_type_no_sharing ty in
-      Typ.Hashtbl.replace type_wrappers ty wrapper_type;
-      auto_type_wrappers := Typ.Set.add wrapper_type !auto_type_wrappers;
+      Cil_datatype.Typ.Hashtbl.replace type_wrappers ty wrapper_type;
+      auto_type_wrappers :=
+        Cil_datatype.Typ.Set.add wrapper_type !auto_type_wrappers;
       (* Treat newly constructed type *)
       let store_current_global = !currentGlobal in
-      ignore (visitFramacGlobal (self:>frama_c_visitor) wrapper_def);
+      ignore (Cil.visitCilGlobal (self:>Cil.cilVisitor) wrapper_def);
       currentGlobal := store_current_global;
       (* Return the wrapper type *)
       wrapper_type
@@ -1529,7 +1569,7 @@ object(self)
             None (* Already in a suitable form for Jessie translation. *)
           else if is_array_reference_type ty then
             (* Do not lose the information that this type is a reference *)
-            let size = constant_expr (My_bigint.of_int64 (reference_size ty)) in
+            let size = constant_expr (Integer.of_int64 (reference_size ty)) in
             assert (not (!flatten_multi_dim_array && is_reference_type elemty));
             Some(mkTRefArray(self#new_wrapper_for_type elemty,size,[]))
           else if is_reference_type ty then
@@ -1567,7 +1607,7 @@ object(self)
     match lv with
       | Var _, NoOffset -> lv
       | Var _, _ ->
-          unsupported "cannot handle this lvalue"
+          unsupported "cannot handle this lvalue: %a" Printer.pp_lval lv
       | Mem e, NoOffset ->
           begin match self#wrap_type_if_needed (typeOf e) with
             | Some newtyp ->
@@ -1584,9 +1624,10 @@ object(self)
               | Some newtyp ->
                   let newfi = get_unique_field (pointed_type newtyp) in
                   let newlv =
-                    if is_array_reference_type newtyp then
+                    if Cil.isArrayType (direct_pointed_type newtyp) then
                       lv
-                    else
+                    else (* The array has been directly decayed into a pointer
+                            Use pointer arithmetic instead of index. *)
                       (Mem
                          (new_exp ~loc:e.eloc
                             (BinOp(PlusPI,e,ie,newtyp))),
@@ -1600,14 +1641,13 @@ object(self)
 
   (* Usual methods in visitor interface. *)
 
-  inherit Visitor.generic_frama_c_visitor
-    (Project.current ()) (Cil.inplace_visit ()) as super
+  inherit Visitor.frama_c_inplace
 
-  method vfile _ =
+  method! vfile _ =
     Common.struct_type_for_void := self#new_wrapper_for_type voidType;
     DoChildren
 
-  method vtype ty =
+  method! vtype ty =
     let ty = match self#wrap_type_if_needed ty with
       | Some newty -> newty
       | None -> ty
@@ -1618,10 +1658,10 @@ object(self)
     else
       ChangeTo ty
 
-  method vglob_aux =
+  method! vglob_aux =
     let retype_return v =
       let retyp = getReturnType v.vtype in
-      let newtyp = visitFramacType (self:>frama_c_visitor) retyp in
+      let newtyp = Cil.visitCilType (self:>Cil.cilVisitor) retyp in
       if newtyp != retyp then setReturnTypeVI v newtyp
     in
     function
@@ -1643,11 +1683,11 @@ object(self)
       | GCompTagDecl _ | GEnumTag _ | GEnumTagDecl _
       | GAsm _ | GPragma _ | GText _  -> SkipChildren
 
-  method vlval lv =
+  method! vlval lv =
     ChangeDoChildrenPost (lv, self#postaction_lval)
 
-  method vterm_lval =
-    do_on_term_lval (None,Some self#postaction_lval)
+  method! vterm_lval t =
+    do_on_term_lval (None,Some self#postaction_lval) t
 
 end
 
@@ -1665,17 +1705,13 @@ class removeUselessCasts =
     match (stripInfo etop).enode with
       | CastE(ty,e) ->
           let ety = typeOf e in
-          if isPointerType ty && isPointerType ety then
-            (* Ignore type qualifiers *)
-            let tysig =
-              typeSig (typeRemoveAttributes ["const";"volatile"]
-                         (unrollType (pointed_type ty)))
-            in
-            let etysig =
-              typeSig (typeRemoveAttributes ["const";"volatile"]
-                         (unrollType (pointed_type ety)))
-            in
-            if Cilutil.equals tysig etysig then e else etop
+          if isPointerType ty && isPointerType ety &&
+            Cil_datatype.Typ.equal
+              (Cil.typeDeepDropAttributes
+                 ["const"; "volatile"] (pointed_type ty))
+              (Cil.typeDeepDropAttributes
+                 ["const"; "volatile"] (pointed_type ety))
+          then e
           else etop
       | _ -> etop
   in
@@ -1684,19 +1720,16 @@ class removeUselessCasts =
       | TCastE(ty,t) ->
           if isPointerType ty && Logic_utils.isLogicPointer t then
             (* Ignore type qualifiers *)
-            let tysig =
-              Logic_utils.type_sig_logic (unrollType (pointed_type ty))
-            in
-            let ttysig =
+            let pty = Cil.typeDeepDropAllAttributes (pointed_type ty) in
+            let ptty =
               match t.term_type with
                   Ctype tty ->
-                    if isPointerType tty then
-                      Logic_utils.type_sig_logic (unrollType (pointed_type tty))
-                    else
-                      Logic_utils.type_sig_logic (unrollType (element_type tty))
-                | ty -> fatal "Not a pointer type '%a'" d_logic_type ty
+                    if isPointerType tty then pointed_type tty 
+                    else element_type tty
+                | ty -> fatal "Not a pointer type '%a'"
+                  Cil_datatype.Logic_type.pretty ty
             in
-            if Cilutil.equals tysig ttysig then
+            if Cil_datatype.Typ.equal pty ptty then
               if Logic_utils.isLogicPointerType t.term_type then t
               else
                 (match t.term_node with
@@ -1705,7 +1738,7 @@ class removeUselessCasts =
                    | _ ->
                        fatal
                          "Unexpected array expression casted into pointer: %a"
-                         d_term t
+                         Cil_datatype.Term.pretty t
                 )
             else term
           else term
@@ -1715,9 +1748,9 @@ object
 
   inherit Visitor.frama_c_inplace
 
-  method vexpr e = ChangeDoChildrenPost (preaction_expr e, fun x -> x)
+  method! vexpr e = ChangeDoChildrenPost (preaction_expr e, fun x -> x)
 
-  method vterm t = ChangeDoChildrenPost (preaction_term t, fun x -> x)
+  method! vterm t = ChangeDoChildrenPost (preaction_term t, fun x -> x)
 
 end
 
@@ -1729,7 +1762,7 @@ let remove_useless_casts file =
 (* Translate union fields into structures                                    *)
 (*****************************************************************************)
 
-let generated_union_types = Typ.Hashtbl.create 0
+let generated_union_types = Cil_datatype.Typ.Hashtbl.create 0
 
 class translateUnions =
   let field_to_equiv_type : typ Cil_datatype.Fieldinfo.Hashtbl.t
@@ -1743,7 +1776,7 @@ class translateUnions =
       mkStructSingleton ~padding tname fname fi.ftype in
     let tdef = GCompTag (mcomp, CurrentLoc.get ()) in
     let tdecl = TComp (mcomp, empty_size_cache (), []) in
-    Typ.Hashtbl.add generated_union_types tdecl ();
+    Cil_datatype.Typ.Hashtbl.add generated_union_types tdecl ();
     Cil_datatype.Fieldinfo.Hashtbl.add field_to_equiv_type fi tdecl;
     fi.ftype <- tdecl;
     tdef
@@ -1760,10 +1793,9 @@ class translateUnions =
   in
 object
 
-  inherit Visitor.generic_frama_c_visitor
-    (Project.current ()) (Cil.inplace_visit ()) as super
+  inherit Visitor.frama_c_inplace
 
-  method vglob_aux = function
+  method! vglob_aux = function
     | GCompTag (compinfo,_) as g when not compinfo.cstruct ->
         let fields = compinfo.cfields in
         let field fi = new_field_type fi in
@@ -1773,10 +1805,10 @@ object
     | GCompTag _ | GType _ | GCompTagDecl _ | GEnumTagDecl _
     | GEnumTag _ | GAsm _ | GPragma _ | GText _ -> SkipChildren
 
-  method voffs off =
+  method! voffs off =
     ChangeDoChildrenPost(off,postaction_offset)
 
-  method vterm_offset =
+  method! vterm_offset =
     do_on_term_offset (None, Some postaction_offset)
 
 end
@@ -1791,10 +1823,9 @@ let translate_unions file =
 class removeArrayAddress =
 object
 
-  inherit Visitor.generic_frama_c_visitor
-    (Project.current ()) (Cil.inplace_visit ()) as super
+  inherit Visitor.frama_c_inplace
 
-  method vexpr e =
+  method! vexpr e =
     let preaction e = match e.enode with
       | AddrOf(Mem ptre,Index(ie,NoOffset)) ->
           let ptrty = typeOf e in
@@ -1803,7 +1834,7 @@ object
     in
     ChangeDoChildrenPost (preaction e, fun x -> x)
 
-  method vterm t =
+  method! vterm t =
     let preaction t = match t.term_node with
       | TAddrOf(TMem ptrt,TIndex(it,TNoOffset)) ->
           { t with term_node = TBinOp (PlusPI, ptrt, it); }
@@ -1824,8 +1855,6 @@ let remove_array_address file =
 (* Normalize the C file for Jessie translation.                              *)
 (*****************************************************************************)
 
-open Pervasives
-
 let normalize file =
   if checking then check_types file;
   (* Retype variables of array type. *)
