@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -34,16 +34,10 @@
 (* Import from Cil *)
 open Cil_types
 open Cil
-open Cilutil
-open Extlib
 module FCAst = Ast
 module FCProject = Project
 (* Import from Why *)
 open Jc
-open Jc_ast
-open Jc_env
-open Jc_fenv
-open Jc_pervasives
 
 (* Utility functions *)
 open Common
@@ -75,6 +69,36 @@ let treat_jessie_no_prolog () =
   Kernel.CppExtraArgs.add ("-D JESSIE_NO_PROLOG")
 *)
 
+let steal_globals () =
+  let vis = object
+    inherit Visitor.frama_c_inplace
+    method! vglob_aux g =
+      match g with
+        | GAnnot (a,_) ->
+            Annotations.remove_global (Annotations.emitter_of_global a) a;
+            Annotations.unsafe_add_global Common.jessie_emitter a;
+            (* SkipChildren is not good, as Annotations.remove_global has
+               removed it from AST: we have to re-insert it...
+            *)
+            ChangeTo [g]
+        | _ -> SkipChildren
+  end 
+  in
+  Visitor.visitFramacFile vis (FCAst.get())
+
+let steal_annots () =
+  let emitter=Common.jessie_emitter in
+  let l = 
+    Annotations.fold_all_code_annot 
+      (fun stmt e ca acc ->
+        Annotations.remove_code_annot e stmt ca;
+        (stmt,ca)::acc)
+      []
+  in
+  List.iter (fun (stmt,ca) -> Annotations.add_code_annot emitter stmt ca) l;
+  steal_globals ();
+  Globals.Functions.iter (Annotations.register_funspec ~emitter ~force:true)
+
 let apply_if_dir_exist name f =
   try
     let d = Unix.opendir name in
@@ -93,14 +117,13 @@ let run () =
   Jessie_options.feedback "Starting Jessie translation";
   (* Work in our own project, initialized by a copy of the main one. *)
   let prj =
-    File.create_project_from_visitor "jessie"
+    File.create_project_from_visitor "jessie" 
       (fun prj -> new Visitor.frama_c_copy prj)
   in
-  FCProject.copy ~selection:(Plugin.get_selection ()) prj;
+  Jessie_options.debug "Project created";
+  FCProject.copy ~selection:(Parameter_state.get_selection ()) prj;
   FCProject.set_current prj;
-
   let file = FCAst.get () in
-
   try
     if file.globals = [] then
       Jessie_options.abort "Nothing to process. There was probably an error before.";
@@ -111,10 +134,15 @@ let run () =
      *)
     ignore (Common.malloc_function ());
     ignore (Common.free_function ());
-
+    Jessie_options.debug  "After malloc and free";
+    if checking then check_types file;
+    steal_annots ();
+    Jessie_options.debug "After steal_annots";
+    if checking then check_types file;
     (* Rewrite ranges in logic annotations by comprehesion *)
-    !Db.Properties.Interp.from_range_to_comprehension
-      (Cil.inplace_visit ()) (FCProject.current ()) file;
+    Jessie_options.debug "from range to comprehension";
+    Rewrite.from_range_to_comprehension
+      (Cil.inplace_visit ()) file;
     if checking then check_types file;
 
     (* Phase 2: C-level rewriting to facilitate analysis *)
@@ -135,8 +163,7 @@ let run () =
     (* Phase 4: various postprocessing steps, still on Cil AST *)
 
     (* Rewrite ranges back in logic annotations *)
-    !Db.Properties.Interp.from_comprehension_to_range
-      (Cil.inplace_visit ()) (FCProject.current ()) file;
+    Rewrite.from_comprehension_to_range (Cil.inplace_visit ()) file;
 
     if Jessie_options.debug_atleast 1 then print_to_stdout file;
 
@@ -207,6 +234,16 @@ let run () =
 	       s);
 	!res
       in
+      let why3_opt =
+	let res = ref "" in
+	Jessie_options.Why3Opt.iter
+	  (fun s ->
+	     res := Format.sprintf "%s%s-why3-opt %S"
+	       !res
+	       (if !res = "" then "" else " ")
+	       s);
+	!res
+      in
       let jc_opt = Jessie_options.JcOpt.get_set ~sep:" " () in
       let debug_opt = if Jessie_options.debug_atleast 1 then " -d " else " " in
       let behav_opt =
@@ -255,14 +292,14 @@ let run () =
       let atp = Jessie_options.Atp.get () in
       let jessie_opt =
 	match atp with
-	  | "why3" | "why3ide" -> ""
-          | "why3ml" -> "-why3ml"
+	  | "why3" -> ""
+          | "why3ml" | "why3ide" | "why3replay" -> "-why3ml"
 	  | _ -> "-why-opt -split-user-conj"
       in
       let cmd =
 	make_command
 	  [ env_opt; jessie_cmd; jessie_opt ;
-	    verbose_opt; why_opt; jc_opt; debug_opt; behav_opt;
+	    verbose_opt; why_opt; why3_opt; jc_opt; debug_opt; behav_opt;
 	    "-locs"; locname; filename ]
       in
       (*      Format.eprintf "CMD=%S" cmd; *)
@@ -289,21 +326,12 @@ let run () =
 let run_and_catch_error () =
   try run ()
   with
-    | Unsupported _ as e ->
-	warn_general "Unsupported feature(s).@\nJessie plugin can not be used on your code." ;
-	if Jessie_options.debug_atleast 1 then raise e else ()
-    | NotImplemented _ ->
-	warn_general "Not implemented feature(s). \
-Please submit `feature request' report."
-    (*| Assert_failure(file,a,b) ->
-	fatal
-	  "Unexpected failure.@\nPlease submit bug report (Ref. \"%s:%d:%d\")."
-	  file a b
-      | exn ->
-	fatal
-	  "Unexpected exception.@\nPlease submit bug report (Ref. \"%s\")."
-	  (Printexc.to_string exn)
-*)
+    | Unsupported _ ->
+	Jessie_options.error "Unsupported feature(s).@\n\
+           Jessie plugin can not be used on your code."
+    | Log.FeatureRequest (_,s) ->
+	Jessie_options.error "Unimplemented feature: %s." s
+
 let run_and_catch_error =
   Dynamic.register
     "run_analysis"
