@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -34,10 +34,8 @@
 (* Import from Cil *)
 open Cil_types
 open Cil
-open Cilutil
 open Cil_datatype
 open Ast_info
-open Extlib
 
 open Visitor
 
@@ -53,10 +51,9 @@ class collectIntField
   (cast_field_to_type : typ Cil_datatype.Fieldinfo.Hashtbl.t) =
 object
 
-  inherit Visitor.generic_frama_c_visitor
-    (Project.current ()) (Cil.inplace_visit ()) as super
+  inherit Visitor.frama_c_inplace
 
-  method vexpr e = match e.enode with
+  method! vexpr e = match e.enode with
     | CastE(ty,e) ->
 	if isPointerType ty then
 	  match (stripCastsAndInfo e).enode with
@@ -94,10 +91,9 @@ class retypeIntField
   in
 object
 
-  inherit Visitor.generic_frama_c_visitor
-    (Project.current ()) (Cil.inplace_visit ()) as super
+  inherit Visitor.frama_c_inplace
 
-  method vglob_aux = function
+  method! vglob_aux = function
     | GCompTag (compinfo,_) ->
 	let fields = compinfo.cfields in
 	let field fi =
@@ -108,7 +104,7 @@ object
 	DoChildren
     | _ -> DoChildren
 
-  method vterm =
+  method! vterm =
     do_on_term (None,Some postaction_expr)
 
 end
@@ -134,8 +130,8 @@ module UnionFind
 	val equal : t -> t -> bool
 	val prefer : t -> t -> int
     end)
-  (ElemSet : Set.S with type elt = Elem.t)
-  (ElemTable : Hashtbl.S with type key = Elem.t) =
+  (ElemSet : Datatype.Set with type elt = Elem.t)
+  (ElemTable : Datatype.Hashtbl with type key = Elem.t) =
 struct
 
   let table = ElemTable.create 73
@@ -308,10 +304,9 @@ class createStructHierarchy =
   in
 object
 
-  inherit Visitor.generic_frama_c_visitor
-    (Project.current ()) (Cil.inplace_visit ()) as super
+  inherit Visitor.frama_c_inplace
 
-  method vexpr e = match e.enode with
+  method! vexpr e = match e.enode with
     | CastE(ty,e) when isPointerType ty ->
 	let enocast = stripCastsAndInfo e in
 	let ety = typeOf enocast in
@@ -326,10 +321,7 @@ end
 class exploitStructHierarchy =
 object
 
-  inherit Visitor.generic_frama_c_visitor
-    (Project.current ()) (Cil.inplace_visit ()) as super
-
-
+  inherit Visitor.frama_c_inplace
 
 end
 
