@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -389,6 +389,7 @@ let create_logic_fun pos fi =
     List.map (create_var pos) fi.java_logic_info_parameters;
   nfi.jc_logic_info_labels <-
     List.map tr_logic_label fi.java_logic_info_labels;
+  (* eprintf "adding symbol %s in logics_table@." fi.java_logic_info_name; *)
   Hashtbl.add logics_table fi.java_logic_info_tag nfi;
   nfi
 
@@ -815,6 +816,7 @@ let rec assertion ?(reg=false) a =
 	    ~expr_then: (assertion a1)
 	    ~expr_else: (assertion a2)
 	    ()
+      | JAfresh t -> mkfresh (term t) ()
   in
   let a' = new pexpr ~pos:a.java_assertion_loc a'#node in
   if reg then locate a.java_assertion_loc a' else a'
@@ -1465,7 +1467,8 @@ let rec expr ?(reg=false) e =
 	  let si = get_class ci.constr_info_class.class_info_name in
 	  let ty = JCTpointer(JCtag(si, []), Some num_zero, Some num_zero) in
 	  let this = Jc_pervasives.var ~formal:true ty "this" in
-	  let tt = Jc_pervasives.var ~formal:true Jc_pervasives.unit_type "tt" in
+	  let tt = Jc_pervasives.var ~formal:true
+            Jc_pervasives.unit_type "_tt" in
 	  let args =
             (mkvar ~name:(var_name this) ()) :: List.map expr args in
           mklet
@@ -1594,7 +1597,7 @@ let loop_annot annot =
   let v = variant annot.loop_var in
   ([new identifier "default"],Some invariant,None)::behs_inv, v
 
-let behavior (id,assumes,throws,assigns,ensures) =
+let behavior (id,assumes,throws,assigns,allocates,ensures) =
   mkbehavior
     ~pos: (fst id)
     ~name: (snd id)
@@ -1610,6 +1613,14 @@ let behavior (id,assumes,throws,assigns,ensures) =
             ~locations:(List.map (location (Some LabelPre)) a)
             ())
        assigns)
+    ?allocates:
+    (Option_misc.map
+       (fun (pos,a) ->
+          mkassigns
+            ~pos
+            ~locations:(List.map (location (Some LabelHere)) a)
+            ())
+       allocates)
     ?assumes: (Option_misc.map assertion assumes)
     ~ensures: (reg_assertion ensures)
     ()
