@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -144,37 +144,39 @@ class term_var ?(pos = Loc.dummy_position) ?(mark="") v =
   term ~pos ~typ:v.jc_var_info_type ~mark ~region:v.jc_var_info_region
     (JCTvar v)
 
-class location ?(pos = Loc.dummy_position) ?label ?region node =
+class location ?(pos = Loc.dummy_position) ~typ ?label ?region node =
   let region = 
     match region with None -> dummy_region | Some region -> region 
   in
 object
+  inherit typed typ
   inherit regioned region
   inherit labeled label
   inherit [location_node] node_positioned ~pos node
 end
 
 (* ignore argument's label *)
-class location_with ?pos ?label ?region ~node t =
+class location_with ?pos ~typ ?label ?region ~node t =
   let pos = match pos with None -> t#pos | Some pos -> pos in
   let region = match region with None -> t#region | Some region -> region in
-  location ~pos ?label ~region node
+  location ~pos ~typ ?label ~region node
 
-class location_set ?(pos = Loc.dummy_position) ?label ?region node =
+class location_set ?(pos = Loc.dummy_position) ~typ ?label ?region node =
   let region = 
     match region with None -> dummy_region | Some region -> region 
   in
 object
+  inherit typed typ
   inherit regioned region
   inherit labeled label
   inherit [location_set_node] node_positioned ~pos node
 end
 
 (* ignore argument's label *)
-class location_set_with ?pos ?label ?region ~node t =
+class location_set_with ?pos ~typ ?label ?region ~node t =
   let pos = match pos with None -> t#pos | Some pos -> pos in
   let region = match region with None -> t#region | Some region -> region in
-  location_set ~pos ?label ~region node
+  location_set ~pos ~typ ?label ~region node
 
 class expr ?(pos = Loc.dummy_position) ~typ ?(mark="") ?region
   ?original_type node =
@@ -333,6 +335,7 @@ either with (~expr1 AND ~expr2) OR ~list only."
   let mkexists = mkquantifier ~quantifier:Exists
   let mkold ~expr = mk ~node:(JCPEold expr)
   let mkat ~expr ~label = mk ~node:(JCPEat(expr, label))
+  let mkfresh ~expr = mk ~node:(JCPEfresh(expr))
   let mkoffset ~kind ~expr = mk ~node:(JCPEoffset(kind, expr))
   let mkoffset_min = mkoffset ~kind:Offset_min
   let mkoffset_max = mkoffset ~kind:Offset_max
@@ -445,21 +448,21 @@ module PDecl = struct
   let mkint_model_def ~value = mk ~node:(JCDint_model value)
 
   let mkbehavior ?(pos = Loc.dummy_position) ~name ?throws ?assumes ?requires
-      ?assigns ?(ensures = mkboolean ~value:true ()) () =
-    (pos, name, throws, assumes, requires, assigns, ensures)
+      ?assigns ?allocates ?(ensures = mkboolean ~value:true ()) () =
+    (pos, name, throws, assumes, requires, assigns, allocates, ensures)
 
   let mkrequires_clause expr = JCCrequires expr
 
   let mkdecreases_clause ?measure expr = JCCdecreases(expr,measure)
 
   let mkbehavior_clause ?(pos = Loc.dummy_position) ~name ?throws ?assumes ?requires
-      ?assigns ?(ensures = mkboolean ~value:true ()) () =
-      JCCbehavior (mkbehavior ~pos ~name ?throws ?assumes ?requires ?assigns ~ensures ())
+      ?assigns ?allocates ?(ensures = mkboolean ~value:true ()) () =
+      JCCbehavior (mkbehavior ~pos ~name ?throws ?assumes ?requires ?assigns ?allocates ~ensures ())
 
-  let mkbehavior_clause_with ?pos ?name ?throws ?assumes ?requires ?assigns ?ensures =
+  let mkbehavior_clause_with ?pos ?name ?throws ?assumes ?requires ?assigns ?allocates ?ensures =
     function
       | JCCbehavior(pos', name', throws', assumes', requires', assigns',
-                    ensures') ->
+                    allocates', ensures') ->
           JCCbehavior(
             oo pos pos',
             oo name name',
@@ -467,6 +470,7 @@ module PDecl = struct
             oo assumes assumes',
             oo requires requires',
             oo assigns assigns',
+            oo allocates allocates',
             oo ensures ensures'
           )
       | _ -> raise (Invalid_argument "mkbehavior_with")
@@ -476,7 +480,7 @@ module PDecl = struct
   let mktag_invariant ~name ~var ~body = name, var, body
 
   let behavior_ensures = function
-    | JCCbehavior(_, _, _, _, _, _, e) -> e
+    | JCCbehavior(_, _, _, _, _, _, _, e) -> e
     | _ -> raise (Invalid_argument "behavior_ensures")
 end
 
