@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -156,12 +156,14 @@ class term_var :
 
 class location :
   ?pos:Loc.position ->
+  typ:jc_type ->
   ?label:label ->
   ?region:region ->
   location_node ->
   object
     val mutable r : region
     method pos : Loc.position
+    method typ : jc_type
     method node : location_node
     method region : region
     method set_region : region -> unit
@@ -171,6 +173,7 @@ class location :
 
 class location_with :
   ?pos:Loc.position ->
+  typ:jc_type ->
   ?label:label ->
   ?region:region ->
   node:location_node ->
@@ -179,12 +182,14 @@ class location_with :
 
 class location_set :
   ?pos:Loc.position ->
+  typ:jc_type ->
   ?label:label ->
   ?region:region ->
   location_set_node ->
   object
     val mutable r : region
     method pos : Loc.position
+    method typ : jc_type
     method node : location_set_node
     method region : region
     method set_region : region -> unit
@@ -194,6 +199,7 @@ class location_set :
 
 class location_set_with :
   ?pos:Loc.position ->
+  typ:jc_type ->
   ?label:label ->
   ?region:region ->
   node:location_set_node ->
@@ -407,6 +413,9 @@ module PExpr :
     val mkoffset_max :
       expr:pexpr -> ?pos:Loc.position -> unit -> pexpr
 
+    val mkfresh :
+      expr:pexpr -> ?pos:Loc.position -> unit -> pexpr
+
     val mkif :
       condition:pexpr ->
       expr_then:pexpr ->
@@ -666,6 +675,7 @@ module PDecl :
       ?assumes:pexpr ->
       ?requires:pexpr ->
       ?assigns:Loc.position * pexpr list ->
+      ?allocates:Loc.position * pexpr list ->
       ?ensures:pexpr -> unit -> pexpr pbehavior
 
     val mkrequires_clause : 'a -> 'a clause
@@ -679,6 +689,7 @@ module PDecl :
       ?assumes:pexpr ->
       ?requires:pexpr ->
       ?assigns:Loc.position * pexpr list ->
+      ?allocates:Loc.position * pexpr list ->
       ?ensures:pexpr -> unit -> pexpr clause
 
 (*
