@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -54,22 +54,22 @@ type precision_mode = MApprox | MPrecise
 let current_mode = ref MApprox
 
 (* Constant memories *)
-let constant_memories = Hashtbl.create 17
+let constant_memories = StringHashtblIter.create 17
 
 (* Constant allocation tables *)
-let constant_alloc_tables = Hashtbl.create 17
+let constant_alloc_tables = StringHashtblIter.create 17
 
 (* Constant allocation tables *)
-let constant_tag_tables = Hashtbl.create 17
+let constant_tag_tables = StringHashtblIter.create 17
 
 let add_constant_memory (mc,r) =
-  Hashtbl.add constant_memories (memory_name (mc,r)) (mc,r)
+  StringHashtblIter.add constant_memories (memory_name (mc,r)) (mc,r)
 
 let add_constant_alloc_table (ac,r) =
-  Hashtbl.add constant_alloc_tables (alloc_table_name (ac,r)) (ac,r)
+  StringHashtblIter.add constant_alloc_tables (alloc_table_name (ac,r)) (ac,r)
 
 let add_constant_tag_table (vi,r) =
-  Hashtbl.add constant_tag_tables (tag_table_name (vi,r)) (vi,r)
+  StringHashtblIter.add constant_tag_tables (tag_table_name (vi,r)) (vi,r)
 
 (* Transposition for calls *)
 
@@ -909,7 +909,7 @@ let rec location_of_expr e =
 	  JCLderef(location_set_of_expr e1, LabelHere, fi, e#region)
       | _ -> failwith "No location for expr"
     in
-    Some(new location_with ~node:loc_node e)
+    Some(new location_with ~typ:e#typ ~node:loc_node e)
   with Failure "No location for expr" -> None
 
 and location_set_of_expr e =
@@ -923,7 +923,7 @@ and location_set_of_expr e =
 	JCLSrange(location_set_of_expr e1, t2_opt, t2_opt)
     | _ -> failwith "No location for expr"
   in
-  new location_set_with ~node:locs_node e
+  new location_set_with ~typ:e#typ ~node:locs_node e
 
 let location_set_of_expr e =
   try Some(location_set_of_expr e) with Failure "No location for expr" -> None
@@ -941,7 +941,7 @@ let rec location_of_term t =
              | Some l1 -> JCLat(l1,lab))
       | _ -> failwith "No location for term"
     in
-    Some(new location_with ~node t)
+    Some(new location_with ~typ:t#typ ~node t)
   with Failure "No location for term" -> None
 
 and location_set_of_term t =
@@ -953,7 +953,7 @@ and location_set_of_term t =
     | JCTat (t1,lab) -> JCLSat(location_set_of_term t1,lab)
     | _ -> failwith "No location for term"
   in
-  new location_set_with ~node:locs_node t
+  new location_set_with ~typ:t#typ ~node:locs_node t
 
 
 (* last location can be mutated *)
@@ -1111,6 +1111,10 @@ let single_assertion ef a =
     match a#label with None -> LabelHere | Some lab -> lab
   in
   match a#node with
+    | JCAfresh(t) ->
+        let ac = tderef_alloc_class ~type_safe:true t in
+	true,
+	add_alloc_effect lab ef (ac,t#region)
     | JCAinstanceof(t,lab,st) ->
 	true,
 	add_tag_effect lab ef (struct_root st,t#region)
@@ -1160,53 +1164,85 @@ let single_assertion fef a =
   let cont,ef = single_assertion fef.jc_reads a in
   cont,{ fef with jc_reads = ef }
 
-let single_location ~in_assigns fef loc =
+type assign_alloc_clause = Assigns | Allocates | Reads
+
+let rec single_location ~in_clause fef 
+    (loc : Jc_fenv.logic_info Jc_ast.location) =
   let lab =
     match loc#label with None -> LabelHere | Some lab -> lab
   in
   let fef = match loc#node with
     | JCLvar v ->
-	if v.jc_var_info_assigned then
-	  if v.jc_var_info_static then
-	    if in_assigns then
-	      add_global_writes lab fef v
-	    else
-	      add_global_reads lab fef v
+        let fef =
+	  if v.jc_var_info_assigned then
+	    if v.jc_var_info_static then
+              begin
+	        match in_clause with
+                  | Assigns -> add_global_writes lab fef v
+                  | Allocates -> 
+                      let ac = lderef_alloc_class ~type_safe:true loc in
+	              add_alloc_writes lab fef (ac,loc#region)
+	          | Reads -> add_global_reads lab fef v
+              end
+	    else fef
 	  else fef
-	else fef
+        in
+        fef
+        
     | JCLderef(locs,lab,fi,_r) ->
 	let add_mem ~only_writes fef mc =
-	  if in_assigns then
-	    let fef = add_memory_writes lab fef (mc,locs#region) in
-	    if only_writes then fef else
+	  match in_clause with
+            | Assigns ->
+	      let fef = add_memory_writes lab fef (mc,locs#region) in
+	      if only_writes then fef else
 	      (* Add effect on allocation table for [not_assigns] predicate *)
-	      let ac = alloc_class_of_mem_class mc in
-	      add_alloc_reads lab fef (ac,locs#region)
-	  else
-	    if only_writes then fef else
-	      add_memory_reads lab fef (mc,locs#region)
+	        let ac = alloc_class_of_mem_class mc in
+	        add_alloc_reads lab fef (ac,locs#region)
+            | Allocates ->
+                (* wrong !
+                  let fef =
+                  if only_writes then fef else
+	            let ac = alloc_class_of_mem_class mc in
+	            add_alloc_writes lab fef (ac,locs#region)
+                in
+                *)
+                fef
+            | Reads ->
+	        if only_writes then fef else
+	          add_memory_reads lab fef (mc,locs#region)
 	in
 	let mc,ufi_opt = lderef_mem_class ~type_safe:true locs fi in
 	let fef = add_mem ~only_writes:false fef mc in
-	begin match mc,ufi_opt with
-	  | JCmem_field _fi, Some ufi ->
-	      let mems = overlapping_union_memories ufi in
-	      List.fold_left (add_mem ~only_writes:true) fef mems
-	  | JCmem_field _, None
-	  | JCmem_plain_union _, _
-	  | JCmem_bitvector, _ -> fef
-	end
+	let fef =
+          begin match mc,ufi_opt with
+	    | JCmem_field _fi, Some ufi ->
+	        let mems = overlapping_union_memories ufi in
+	        List.fold_left (add_mem ~only_writes:true) fef mems
+	    | JCmem_field _, None
+	    | JCmem_plain_union _, _
+	    | JCmem_bitvector, _ -> fef
+	  end
+        in
+        fef
     | JCLderef_term(t1,fi) ->
 	let add_mem ~only_writes fef mc =
-	  if in_assigns then
-	    let fef = add_memory_writes lab fef (mc,t1#region) in
-	    if only_writes then fef else
+	  match in_clause with
+            | Assigns ->
+	      let fef = add_memory_writes lab fef (mc,t1#region) in
+	      if only_writes then fef else
 	      (* Add effect on allocation table for [not_assigns] predicate *)
-	      let ac = alloc_class_of_mem_class mc in
-	      add_alloc_reads lab fef (ac,t1#region)
-	  else
-	    if only_writes then fef else
-	      add_memory_reads lab fef (mc,t1#region)
+	        let ac = alloc_class_of_mem_class mc in
+	        add_alloc_reads lab fef (ac,t1#region)
+            | Allocates -> 
+(* wrong !
+	      if only_writes then fef else
+	        let ac = alloc_class_of_mem_class mc in
+	        add_alloc_writes lab fef (ac,t1#region)
+*)
+                fef
+	    | Reads ->
+	        if only_writes then fef else
+	          add_memory_reads lab fef (mc,t1#region)
 	in
 	let mc,ufi_opt = tderef_mem_class ~type_safe:true t1 fi in
 	let fef = add_mem ~only_writes:false fef mc in
@@ -1218,7 +1254,9 @@ let single_location ~in_assigns fef loc =
 	  | JCmem_plain_union _, _
 	  | JCmem_bitvector, _ -> fef
 	end
-    | JCLat(_loc,_lab) -> fef
+    | JCLat(loc,lab) -> 
+      assert (loc#label = Some lab);
+      snd (single_location ~in_clause fef loc)
   in true, fef
 
 let rec single_location_set fef locs =
@@ -1241,9 +1279,12 @@ let rec single_location_set fef locs =
     | JCLSat(locs,_) -> snd (single_location_set fef locs)
   in true, fef
 
-let location ~in_assigns fef loc =
-  Jc_iterators.fold_rec_location single_term
-    (single_location ~in_assigns) single_location_set fef loc
+let location ~in_clause fef loc =
+  let fef =
+    Jc_iterators.fold_rec_location single_term
+      (single_location ~in_clause) single_location_set fef loc
+  in
+  fef
 
 
 (******************************************************************************)
@@ -1253,7 +1294,7 @@ let location ~in_assigns fef loc =
 let rec expr fef e =
   Jc_iterators.fold_rec_expr_and_term_and_assertion
     single_term single_assertion
-    (single_location ~in_assigns:true) single_location_set
+    (single_location ~in_clause:Assigns) single_location_set
     (fun (fef : fun_effect) e -> match e#node with
        | JCEvar v ->
 	   true,
@@ -1392,11 +1433,13 @@ let rec expr fef e =
 		  add_memory_writes LabelHere fef (mc,e#region)
 	       ) fef all_mems
 	   in
+(**)
 	   let fef =
 	     List.fold_left
 	       (fun fef ac -> add_alloc_writes LabelHere fef (ac,e#region))
 	       fef all_allocs
 	   in
+(**)
 	   true,
 	   List.fold_left
 	     (fun fef vi -> add_tag_writes LabelHere fef (vi,e#region))
@@ -1410,7 +1453,8 @@ let rec expr fef e =
 	   (* Assert the invariants of the structure
 	      => need the reads of the invariants *)
 	   let (_, invs) =
-	     Hashtbl.find Jc_typing.structs_table st.jc_struct_info_name
+	     StringHashtblIter.find
+               Jc_typing.structs_table st.jc_struct_info_name
 	   in
 	   let fef =
 	     List.fold_left
@@ -1491,17 +1535,41 @@ let rec expr fef e =
        | JCEloop _ | JCElet _ | JCEassert _ | JCEcontract _ | JCEblock _
        | JCEconst _  | JCEshift _ | JCEif _ | JCErange_cast _
        | JCEreal_cast _ | JCEunary _ | JCEaddress _ | JCEbinary _
-       | JCEreturn_void  | JCEreturn _ | JCEbitwise_cast _ | JCEbase_block _ ->
+       | JCEreturn_void  | JCEreturn _ | JCEbitwise_cast _ | JCEbase_block _ | JCEfresh _ ->
 	   true, fef
     ) fef e
 
 let behavior fef (_pos,_id,b) =
-  let fef =
-    Jc_iterators.fold_rec_behavior single_term single_assertion
-      (single_location ~in_assigns:true) single_location_set fef b
+  let ita = 
+    Jc_iterators.fold_rec_term_and_assertion single_term single_assertion 
   in
+  let itl = 
+    Jc_iterators.fold_rec_location single_term
+      (single_location ~in_clause:Assigns) single_location_set
+  in
+  let itl_alloc = 
+    Jc_iterators.fold_rec_location single_term
+      (single_location ~in_clause:Allocates) single_location_set
+  in
+  let fef = Option_misc.fold_left ita fef b.jc_behavior_assumes in
+  let fef =
+    Option_misc.fold_left
+      (fun fef (_,locs) -> List.fold_left itl fef locs)
+      fef b.jc_behavior_assigns
+  in
+  let fef =
+    Option_misc.fold_left
+      (fun fef (_,locs) -> 
+        List.fold_left itl_alloc fef locs)
+      fef b.jc_behavior_allocates
+  in
+  let fef = ita fef b.jc_behavior_ensures in
+  let fef = ita fef b.jc_behavior_free_ensures in
   Option_misc.fold_left add_exception_effect fef b.jc_behavior_throws
 
+
+
+
 let spec fef s =
   let fef =
     List.fold_left behavior fef
@@ -1510,6 +1578,9 @@ let spec fef s =
   let fef = { fef with jc_reads = assertion fef.jc_reads s.jc_fun_requires } in
   { fef with jc_reads = assertion fef.jc_reads s.jc_fun_free_requires }
 
+
+
+
 (* Type invariant added to precondition for pointer parameters with bounds.
    Therefore, add allocation table to reads. *)
 let parameter fef v =
@@ -1575,14 +1646,14 @@ let effects_from_decl fi ax_effects acc d =
 
 let effects_from_axiomatic fi ax acc =
   try
-    let l = Hashtbl.find Jc_typing.axiomatics_table ax in
+    let l = StringHashtblIter.find Jc_typing.axiomatics_table ax in
     let ef = List.fold_left axiomatic_decl_effect empty_effects l.Jc_typing.axiomatics_decls in
     List.fold_left (effects_from_decl fi ef) acc l.Jc_typing.axiomatics_decls
   with Not_found -> assert false
 
 let logic_fun_effects f =
   let f,ta =
-    Hashtbl.find Jc_typing.logic_functions_table f.jc_logic_info_tag
+    IntHashtblIter.find Jc_typing.logic_functions_table f.jc_logic_info_tag
   in
   let ef = f.jc_logic_info_effects in
   let ef = match ta with
@@ -1606,7 +1677,7 @@ let logic_fun_effects f =
     | JCReads loclist ->
 	List.fold_left
 	  (fun ef loc ->
-	     let fef = location ~in_assigns:false empty_fun_effect loc in
+	     let fef = location ~in_clause:Reads empty_fun_effect loc in
 	     ef_union ef fef.jc_reads
 	  ) ef loclist
   in
@@ -1616,7 +1687,7 @@ let logic_fun_effects f =
 
 let fun_effects f =
   let (f,_pos,s,e_opt) =
-    Hashtbl.find Jc_typing.functions_table f.jc_fun_info_tag
+    IntHashtblIter.find Jc_typing.functions_table f.jc_fun_info_tag
   in
   let fef = f.jc_fun_info_effects in
   let fef = spec fef s in
