@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -237,7 +237,7 @@ let frame_between ftin notin labels =
 
 
 let compute_predicate_framed () =
-  let aux tag (pi, info,params1,params2) =
+  let aux tag (pi, info,params1,params2,kind) =
       test_correct_logic info;
       let params1 = List.map mk_tvar params1 in
       let params2 = List.map mk_tvar params2 in
@@ -250,17 +250,21 @@ let compute_predicate_framed () =
           let lab = Jc_envset.LogicLabelSet.choose labs in
           let notin = NotIn.from_memory false (mem,lab) in
           let app = app_in_logic info params1 label1 notin in
-          let app1 = frame_between app notin pi.jc_logic_info_labels in
           let app2 = app_in_logic info params2 label2 notin in
           let app2 = MyBag.make_jc_sub [app2;app] in
-          app2::app1::acc) mems [] in
+          match kind with
+            | `Frame ->
+              let app1 = frame_between app notin pi.jc_logic_info_labels in
+              app2::app1::acc
+            | `Sub   -> app2::acc
+        ) mems [] in
       let ass = new assertion (JCAand apps) in
-      Hashtbl.replace Jc_typing.logic_functions_table tag
+      IntHashtblIter.replace Jc_typing.logic_functions_table tag
         (pi,JCAssertion ass) in
   Hashtbl.iter aux Jc_typing.pragma_gen_frame
 
 let user_predicate_code queue id kind pred =
-  let (logic,_) = Hashtbl.find Jc_typing.logic_functions_table id in
+  let (logic,_) = IntHashtblIter.find Jc_typing.logic_functions_table id in
   Jc_options.lprintf "Generate code of %s with %i params@." 
     logic.jc_logic_info_name (List.length pred);
   let code = fold_exp (separation_between queue kind) [] pred in
@@ -284,7 +288,8 @@ let user_predicate_code queue id kind pred =
       MyBag.make_jc_disj [fapp;gapp] in
   let code = List.map trad_code code in
   let code = new assertion (JCAand code) in
-  Hashtbl.replace Jc_typing.logic_functions_table id (logic,JCAssertion code)
+  IntHashtblIter.replace
+    Jc_typing.logic_functions_table id (logic,JCAssertion code)
 
 let pragma_gen_sep = Jc_typing.pragma_gen_sep
 
@@ -1368,22 +1373,28 @@ sig
 
   type define = NotIn.t -> why_decl list -> why_decl list -> why_decl list
 
-  val define_In : define
-  val define_disj : define
+  val define_In    : define
+  val define_disj  : define
   val define_frame_between : define
   val define_x_sub : define
   val define_sub_x : define
+  val define_sub   : define
   
   val def_frame_in :  string -> NotIn.t -> (string * Output.logic_type) list ->
     Output.why_decl list -> NotIn.t list -> Output.why_decl list
-  val def_frame : string -> bool -> (string * Output.logic_type) list ->
+  val def_frame : string -> bool -> string ->
+    (string * Output.logic_type) list ->
     Output.why_decl list -> NotIn.t list -> Output.why_decl list
-  val in_for_in : string -> (string * Output.logic_type) list ->
-    NotIn.t -> (Jc_fenv.logic_info * (string * 'k) list) list ->
-    Output.why_decl list -> Output.why_decl list
-  val disj_for_in : string -> (string * Output.logic_type) list ->
-    NotIn.t -> (Jc_fenv.logic_info * (string * 'k) list) list ->
-    Output.why_decl list -> Output.why_decl list
+
+  type for_in = string -> (string * logic_type) list ->
+    NotIn.t -> (Jc_fenv.logic_info * (string * logic_type) list) list ->
+    why_decl list -> why_decl list
+
+  val in_for_in    : for_in
+  val disj_for_in  : for_in
+  val x_sub_for_in : for_in
+  val sub_x_for_in : for_in
+
 end =
 struct
 
@@ -1391,6 +1402,10 @@ struct
     Output.why_decl list ->
     Output.why_decl list -> Output.why_decl list
 
+  type for_in = string -> (string * logic_type) list ->
+    NotIn.t -> (Jc_fenv.logic_info * (string * logic_type) list) list ->
+    why_decl list -> why_decl list
+
 
   let trad_app s lt = (s,lt)
 
@@ -1406,7 +1421,7 @@ struct
   let pred_extract_effect s lt acc =
     (trad_app s lt)::(List.fold_left term_extract_application acc lt)
 
-  let inductive_extract_effect = 
+  let inductive_extract_effect ass = 
     let rec iee e acc =
       match e with
         | LTrue | LFalse -> acc
@@ -1422,7 +1437,8 @@ struct
     and rq = function
         | LForall (_,_,_,f) -> rq f
         | e -> iee e [] in
-    rq
+    let effects = rq ass in
+    remove_double Pervasives.compare effects
 
   let rec rewrite_inductive_case rw = function
     | LForall (s,ty,_,f) -> LForall(s,ty,[],rewrite_inductive_case rw f)
@@ -1516,7 +1532,7 @@ struct
       | `Var var1,`Var var2 -> LPred(sub_pred,[LVar (fst var1); 
                                                LVar (fst var2)])
       (* This case must be in fact var1 is a singleton *)
-      | `Var _, `App (s,[_;_]) when s = select_name -> LFalse 
+      | `Var _, `App (s,[_;_]) when s = select_name -> assert false
       | `App (s,[_;p]), `Var var when s = select_name ->
         LPred(in_pred,[p;LVar (fst var)])
       | `Var var, `App (s,lt) -> 
@@ -1528,7 +1544,7 @@ struct
         when s1 = select_name && s2 = select_name ->
         make_eq p1 p2
       (* This case must be in fact app2 is a singleton *)
-      | `App _, `App(s2,[_;_]) when s2 = select_name -> LFalse
+      | `App _, `App(s2,[_;_]) when s2 = select_name -> assert false
       | `App(s1,[_;p1]), `App(s2,lt2) when s1 = select_name -> 
         LPred(in_pred,[p1;LApp(in_name notin s2, lt2)])
       | `App(s1,lt1), `App(s2,lt2) ->
@@ -1773,6 +1789,68 @@ struct
       | _ -> Jc_options.lprintf "@[<hov 3>I can't translate that :@\n%a" 
           (Pp.print_list Pp.newline fprintf_why_decl) ta_conv;assert false
 
+
+  let rec define_sub notin ta_conv acc = 
+    match ta_conv with 
+        (* Devrait peut-être utiliser la vrai transformation 
+           d'inductif en 1 unique axiom*)
+      | [Inductive (_,f_name,_,l)] -> 
+        let name = (in_name notin f_name.name) in
+        let var = ("jc_var",NotIn.ty notin) in
+        let gen_case acc (ident,assertion) =
+          let effects = inductive_extract_effect assertion in
+          let effects = 
+            List.filter (fun (s,_) -> not (s = select_name)) effects in
+          let effects = List.filter (mem_are_compatible notin) effects in
+          let rw seff lteff s lt =
+            sub_interp_app (`App (seff,lteff)) notin (`App (s,lt)) in
+          let nb = ref 0 in
+          let rewrite acc (seff,lteff) = 
+            incr nb;
+            let assertion =
+              LForall(fst var,snd var,[],
+                      rewrite_inductive_case (rw seff lteff) assertion) in
+            (ident^(string_of_int !nb), assertion) :: acc in
+          List.fold_left rewrite acc effects in
+        let name = ("Sub"^name) in
+        let l = List.fold_left gen_case [] l in
+        let constr acc (ident,assertion) = 
+          Goal(KAxiom,id_no_loc (name^ident),assertion)::acc in
+        let acc = List.fold_left constr acc l in
+        acc
+      | [Function (_,f_name,params,_,term)] ->
+          let name = in_name notin f_name.name in
+          Jc_options.lprintf "Generate logic notin (fun): %s :@." name;
+          let acc = Logic(false,
+                          id_no_loc name,
+                          params,
+                          NotIn.ty notin)::acc in
+          let axiom_name = "axiom"^"_in_"^(NotIn.mem_name2 notin)^f_name.name in
+          Jc_options.lprintf "Generate axiom notin : %s :@." axiom_name;
+          let var = ("jc_var",NotIn.ty_elt notin) in
+          let interp s lt = 
+            if mem_are_compatible notin (s,lt)
+            then in_interp_app var notin s lt
+            else LFalse in
+          let asser = term_interp_or interp term in
+          let conclu = in_interp_app var notin f_name.name
+            (List.map (fun (x,_) -> LVar x) params) in
+          let asser = make_equiv asser conclu in
+          let params = var::params in
+          let asser = make_forall_list params [] asser in
+          let asser = Goal(KAxiom,id_no_loc axiom_name,asser) in
+          asser::acc
+      | [Logic(_bool,f_name,params,_ltype);
+         Goal(KAxiom,_,ax_asser)] ->
+          let rec extract_term = function
+            | LForall (_,_,_,asser) -> extract_term asser
+            | LPred("eq", [ _; b ]) -> b
+            | _ -> assert false (* constructed by tr_fun_def *) in
+          let term = extract_term ax_asser in
+          let ta_conv = [Function (_bool,f_name,params,_ltype,term)] in
+          define_sub notin ta_conv acc
+      | _ -> Jc_options.lprintf "@[<hov 3>I can't translate that :@\n%a" 
+          (Pp.print_list Pp.newline fprintf_why_decl) ta_conv;assert false
             
   let rec define_x_sub notin ta_conv acc = 
     match ta_conv with 
@@ -1848,8 +1926,6 @@ struct
         let var = ("jc_var",NotIn.ty notin) in
         let gen_case acc (ident,assertion) =
           let effects = inductive_extract_effect assertion in
-          let effects = 
-            List.filter (fun (s,_) -> not (s = select_name)) effects in
           let effects = List.filter (mem_are_compatible notin) effects in
           let rw s lt =
             List.fold_left (fun acc (seff,lteff) ->
@@ -2006,19 +2082,20 @@ struct
       acc notin_updates in
     acc
 
-  let def_frame f_name is_pred params acc notin_updates =
+  (** framed_name must have the same param than f_name *)
+  let def_frame framed_name is_pred f_name params acc notin_updates =
     let gen_framed =
       if is_pred
-      then `Pred (fun params -> LPred(f_name,params))
-      else `Term (fun params -> LApp(f_name,params)) in
+      then `Pred (fun params -> LPred(framed_name,params))
+      else `Term (fun params -> LApp(framed_name,params)) in
     let acc = List.fold_left
       (fun acc notin_update ->
-        (gen f_name f_name gen_framed notin_update params params)::acc) 
+        (gen framed_name f_name gen_framed notin_update params params)::acc) 
       acc notin_updates in
     let acc = List.fold_all_part 
       (fun acc part -> match part with | [] -> acc (* trivial axiom *)
         | notin_updates ->
-          gen_many f_name f_name gen_framed notin_updates params params ::acc)
+          gen_many framed_name f_name gen_framed notin_updates params params ::acc)
       acc notin_updates in
     acc
        
@@ -2055,8 +2132,66 @@ struct
     let axiom_name = name^"_disj_def" in
     Goal(KAxiom,id_no_loc axiom_name,code)::acc          
 
+  let x_sub_for_in f_name args notin framed acc =
+    let name = in_name notin f_name in
+    let var = ("jc_var",NotIn.ty notin) in
+    let conjs = List.map 
+      (fun (f,params) ->
+         (sub_interp_app (`Var var) notin (`App (f.jc_logic_info_name,
+            (List.map (fun (x,_) -> LVar x) params)))))
+      framed in
+    let code = make_and_list conjs in
+    let conclu = sub_interp_app (`Var var) notin (`App (f_name,
+      (List.map (fun (x,_) -> LVar x) args))) in
+    let code = make_equiv code conclu in
+    let code = make_forall_list (var::args) [] code in
+    let axiom_name = name^"_x_sub_def" in
+    Goal(KAxiom,id_no_loc axiom_name,code)::acc          
+
+  let sub_x_for_in f_name args notin framed acc =
+    let name = in_name notin f_name in
+    let var = ("jc_var",NotIn.ty notin) in
+    let conjs = List.map 
+      (fun (f,params) ->
+         (sub_interp_app  (`App (f.jc_logic_info_name,
+            (List.map (fun (x,_) -> LVar x) params)))) notin (`Var var))
+      framed in
+    let code = make_and_list conjs in
+    let conclu = sub_interp_app (`App (f_name,
+      (List.map (fun (x,_) -> LVar x) args))) notin (`Var var) in
+    let code = make_equiv code conclu in
+    let code = make_forall_list (var::args) [] code in
+    let axiom_name = name^"_sub_x_def" in
+    Goal(KAxiom,id_no_loc axiom_name,code)::acc 
+
 
 end
+
+let lemma_disjoint_cases ta_conv acc =
+  match ta_conv with
+    | [Inductive (_,f_name,params,l)] ->
+      let rec link_by_implication args cases = function
+        | LForall (s,ty,_,f) ->
+          LForall(s,ty,[],link_by_implication args cases f)
+        | LImpl(f1,f2) -> LImpl(f1,link_by_implication args cases f2)
+        | LPred(s,lt) when f_name.name = s ->
+          let equalities = List.map2 (fun (a1,_) a2 -> make_eq (LVar a1) a2)
+            args lt in
+          LImpl (make_and_list equalities,each_case args cases)
+        | _ -> failwith "Inductive must be forall x,.. Pn -> ... -> P1"
+      and each_case args = function
+        | [] -> LFalse
+        | (_,assertion)::cases -> link_by_implication args cases assertion in
+      let args = List.map (fun (s,ty) -> (s^tmp_suffix,ty)) params in
+      let condition = make_forall_list args [] (each_case args l) in
+      let goal_name = f_name.name^"_disjoint_case" in
+      Goal(KGoal,id_no_loc goal_name,condition)::acc
+    | _ -> Jc_options.lprintf "@[<hov 3>I can't translate that :@\n%a"
+      (Pp.print_list Pp.newline fprintf_why_decl) ta_conv;assert false
+
+
+        
+
     
 let tr_params_usual_model f =
   let usual_params,model_params = tr_params_usual_model_aux f in
@@ -2097,7 +2232,8 @@ let tr_logic_fun_aux f ta acc =
   (* generation of ft *)
   let acc =
     if Jc_options.gen_frame_rule_with_ft 
-    then if Hashtbl.mem pragma_gen_sep f.jc_logic_info_tag
+    then
+      let acc = if Hashtbl.mem pragma_gen_sep f.jc_logic_info_tag
       then begin
         (* use_predicate *)
         let (_,which) = Hashtbl.find pragma_gen_sep f.jc_logic_info_tag in
@@ -2116,7 +2252,14 @@ let tr_logic_fun_aux f ta acc =
         let make_todo acc (todo,notin) =
           match todo with
             | `In ->
-              InDisj.in_for_in f.jc_logic_info_name args notin framed acc
+              let fname = f.jc_logic_info_name in
+              let acc = 
+                InDisj.in_for_in fname args notin framed acc in
+              let acc = 
+                InDisj.x_sub_for_in fname args notin framed acc in
+              let acc = 
+                InDisj.sub_x_for_in fname args notin framed acc in
+              acc
             | `Disj ->
               InDisj.disj_for_in f.jc_logic_info_name args notin framed acc in
         let acc = List.fold_left make_todo acc todos in
@@ -2136,13 +2279,17 @@ let tr_logic_fun_aux f ta acc =
            | (`In,_) -> fprintf fmt "in" in
         Jc_options.lprintf "%s asks to generate : %a@." 
           f_name (Pp.print_list Pp.comma print_todo) todos;
+        let acc = if todos = [] then acc else
+            lemma_disjoint_cases fun_def acc in
         let make_todo acc (todo,notin) =
           match todo with
             | `In -> 
                 let acc = InDisj.define_In notin fun_def acc in
                 let acc = InDisj.define_frame_between notin fun_def acc in
+                let acc = InDisj.define_sub notin fun_def acc in
                 let acc = InDisj.define_x_sub notin fun_def acc in
                 let acc = InDisj.define_sub_x notin fun_def acc in
+                let acc = InDisj.define_sub notin fun_def acc in
                 let acc = 
                   InDisj.def_frame_in f_name notin params acc notin_updates in
                 acc
@@ -2151,9 +2298,26 @@ let tr_logic_fun_aux f ta acc =
                 acc in
         let acc = List.fold_left make_todo acc todos in
         let is_pred = f.jc_logic_info_result_type = None in
-        InDisj.def_frame f_name is_pred params acc notin_updates
-      end
-    else acc in 
+        InDisj.def_frame f_name is_pred f_name params acc notin_updates
+      end in
+      if Hashtbl.mem Jc_typing.pragma_gen_same f.jc_logic_info_tag then
+        let f_name = f.jc_logic_info_final_name in
+        let mirror =
+          Hashtbl.find Jc_typing.pragma_gen_same f.jc_logic_info_tag in
+        let mirror_name = mirror.jc_logic_info_final_name in
+        let todos =
+          Hashtbl.find_all predicates_to_generate mirror.jc_logic_info_tag in
+        let filter_notin acc = function
+          | (`In , notin) -> notin::acc
+          | _ -> acc in
+        (* notin_updates is used to create the frames axioms *)
+        let notin_updates = List.fold_left filter_notin [] todos in
+        Jc_options.lprintf "Frame of %s using footprint of %s@."
+          f_name mirror_name;
+        let is_pred = f.jc_logic_info_result_type = None in
+        InDisj.def_frame f_name is_pred mirror_name params acc notin_updates
+      else acc
+    else acc in
   acc
 
 
@@ -2229,6 +2393,6 @@ let tr_logic_fun ft ta acc = tr_logic_fun_aux ft ta acc
 
 (*
   Local Variables: 
-  compile-command: "unset LANG; make -j -C .. bin/jessie.byte"
+  compile-command: "unset LANG; make -C .. bin/jessie.byte"
   End: 
 *)
