@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -55,10 +55,10 @@ let find_struct a =
 (*
   Format.printf "[find_struct] %s@." a;
 *)
-  fst (Hashtbl.find Jc_typing.structs_table a)
+  fst (StringHashtblIter.find Jc_typing.structs_table a)
 
 let find_variant a =
-  Hashtbl.find Jc_typing.roots_table a
+  StringHashtblIter.find Jc_typing.roots_table a
 
 let find_pointer_class a =
   try
@@ -229,6 +229,9 @@ let reset_current_behavior () = current_behavior := None
 let get_current_behavior () =
   match !current_behavior with None -> assert false | Some behav -> behav
 let safety_checking () = get_current_behavior () = "safety"
+let variant_checking () =
+  let behv = get_current_behavior () in
+  behv = "safety" || behv = "variant"
 
 let is_current_behavior id = id = get_current_behavior ()
 
@@ -368,7 +371,7 @@ let tr_var_base_type v =
 let tr_var_type v =
   tr_type ~region:v.jc_var_info_region v.jc_var_info_type
 
-let any_value t =
+let any_value region t =
   match t with
     | JCTnative ty ->
         begin match ty with
@@ -380,7 +383,7 @@ let any_value t =
 	  | Tstring -> make_app "any_string" [void]
         end
     | JCTnull
-    | JCTpointer _ -> make_app "any_pointer" [void]
+    | JCTpointer _ -> make_app ~ty:(tr_type ~region t) "any_pointer" [void]
     | JCTenum ri -> make_app (fun_any_enum ri) [void]
     | JCTlogic _ as ty ->
         let t =
@@ -425,7 +428,7 @@ let any_value' ty' =
     else if is_memory_type ty' then "any_memory"
     else assert false
   in
-  make_app anyfun [void]
+  make_app ~ty:(Base_type ty') anyfun [void]
 
 
 (******************************************************************************)
@@ -717,11 +720,11 @@ let rec transpose_location ~region_assoc ~param_assoc (loc,(mc,rdist)) =
 		JCLderef(locs,lab,fi,r) (* TODO: remove useless lab & r *)
             | JCLat(loc,lab) ->
                 let node = mk_node loc in
-                JCLat(new location_with ~region:rloc ~node loc,lab)
+                JCLat(new location_with ~typ:loc#typ ~region:rloc ~node loc,lab)
 	    | _ -> assert false (* TODO *)
 	  in
           let node = mk_node loc in
-	  let loc = new location_with ~region:rloc ~node loc in
+	  let loc = new location_with ~typ:loc#typ ~region:rloc ~node loc in
 	  Some(PreciseMemory(loc,(mc,rloc)))
 	with NoRegion ->
 	  Some(RawMemory(mc,rloc))
@@ -754,7 +757,7 @@ and transpose_location_set ~region_assoc ~param_assoc locs =
               in JCLSat(locs',lab)
 	  | _ -> assert false (* TODO *)
 	in
-	new location_set_with ~region:rloc ~node locs
+	new location_set_with ~typ:locs#typ  ~region:rloc ~node locs
 
 let transpose_location_set ~region_assoc ~param_assoc locs _w =
   try Some(transpose_location_set ~region_assoc ~param_assoc locs)
@@ -2135,7 +2138,7 @@ let tag_table_arguments ~callee_reads ~callee_writes ~region_assoc =
   in
   (List.map fst writes), (List.map fst reads)
 
-let specialized_functions = Hashtbl.create 0
+let specialized_functions = StringHashtblIter.create 0
 
 let memory_arguments ~callee_reads ~callee_writes ~region_assoc
     ~region_mem_assoc ~param_assoc ~with_body fname =
@@ -2238,7 +2241,7 @@ let memory_arguments ~callee_reads ~callee_writes ~region_assoc
 	     v :: acc, name_assoc, StringMap.add n ndest already_used_names
 	) reads' ([], name_assoc, already_used_names)
     in
-    Hashtbl.add specialized_functions new_fname (fname,name_assoc);
+    StringHashtblIter.add specialized_functions new_fname (fname,name_assoc);
     pre, new_fname, wpointers, rpointers, writes, reads
 
 let global_arguments ~callee_reads ~callee_writes ~region_assoc:_ =
