@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -67,6 +67,8 @@ val logic_variant_of_bitvector : Jc_env.root_info -> string
 
 val safety_checking : unit -> bool
 
+val variant_checking : unit -> bool
+
 val in_current_behavior : < name : string; .. > list -> bool
 
 val in_default_behavior : < name : string; .. > list -> bool
@@ -180,7 +182,7 @@ val ref_term :
     Jc_env.label -> Jc_env.label -> Jc_fenv.term -> Output.term)
   ref
 
-val any_value : Jc_env.jc_type -> Output.expr
+val any_value : Jc_env.region -> Jc_env.jc_type -> Output.expr
 
 val eq_of_enum : Jc_env.enum_info -> string
 
@@ -412,5 +414,5 @@ val read_locals : region_list:Jc_region.RegionTable.key list ->
 
 (** {1 Misc} *)
 
-val specialized_functions : (string, string * string Jc_envset.StringMap.t)
-  Jc_stdlib.Hashtbl.t
+val specialized_functions: 
+  (string * string Jc_envset.StringMap.t) Jc_pervasives.StringHashtblIter.t
