@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2010                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS                                     *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
-(*    Thierry HUBERT, Univ. Paris-sud 11                                  *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -141,6 +141,8 @@ let replace_sub_pexpr e el =
 	JCPEapp(fi,labs,el)
     | JCPEquantifier(q,ty,labs,trigs,_e) ->
 	let e1 = as1 el in JCPEquantifier(q,ty,labs,trigs,e1)
+    | JCPEfresh _e ->
+	let e1 = as1 el in JCPEfresh(e1)
     | JCPEold _e ->
 	let e1 = as1 el in JCPEold(e1)
     | JCPEat(_e,lab) ->
@@ -297,6 +299,7 @@ module PExprAst = struct
       | JCPEthrow(_, e)
       | JCPEpack(e, _)
       | JCPEunpack(e, _)
+      | JCPEfresh e
       | JCPEold e
       | JCPEat(e,_)
       | JCPErange(Some e,None)
@@ -547,6 +550,7 @@ let rec iter_term_and_assertion ft fa a =
 	(* ITerm.iter *) iter_term ft t2
     | JCAapp app ->
 	List.iter (iter_term ft) app.jc_app_args
+    | JCAfresh t1
     | JCAinstanceof(t1,_,_) | JCAbool_term t1 | JCAmutable(t1,_,_) ->
 	iter_term ft t1
     | JCAand al | JCAor al ->
@@ -639,6 +643,7 @@ let rec fold_term_in_assertion f acc a =
 	fold_term f acc t2
     | JCAapp app ->
 	List.fold_left (fold_term f) acc app.jc_app_args
+    | JCAfresh t1
     | JCAinstanceof(t1,_,_) | JCAbool_term t1 | JCAmutable(t1,_,_) ->
 	fold_term f acc t1
     | JCAand al | JCAor al ->
@@ -703,6 +708,7 @@ let rec fold_sub_term_and_assertion itt ita ft fa acc a =
 	itt ft acc t2
     | JCAapp app ->
 	List.fold_left (itt ft) acc app.jc_app_args
+    | JCAfresh t1
     | JCAinstanceof(t1,_,_) | JCAbool_term t1 | JCAmutable(t1,_,_) ->
 	itt ft acc t1
     | JCAand al | JCAor al ->
@@ -820,11 +826,13 @@ let rec fold_behavior ft fa fl fls acc e =
     fold_term fold_term_and_assertion fold_location fold_location_set
     ft fa fl fls acc e
 
+(*
 let rec fold_rec_behavior ft fa fl fls acc e =
   fold_sub_behavior
     fold_rec_term fold_rec_term_and_assertion fold_rec_location
     fold_rec_location_set
     ft fa fl fls acc e 
+*)
 
 (*
 let iter_behavior ft fa fl fls b =
@@ -913,6 +921,8 @@ let rec map_term_in_assertion f a =
 	JCAinstanceof(map_term f t1,lab,st)
     | JCAbool_term t1 ->
 	JCAbool_term(map_term f t1)
+    | JCAfresh t1 ->
+	JCAfresh(map_term f t1)
     | JCAmutable(t1,st,tag) ->
 	JCAmutable(map_term f t1,st,tag)
     | JCAand al ->
@@ -1047,6 +1057,7 @@ let replace_sub_expr e el =
     | None -> None,el 
     | Some _ -> let e1,r = pop el in Some e1,r
   in
+(* dead code
   let rec popn n el = 
     if n > 0 then 
       let e,r = pop el in
@@ -1054,6 +1065,7 @@ let replace_sub_expr e el =
       e :: el1, el2
     else [],el
   in
+*)
   let enode = match e#node with
     | JCEconst _ | JCEvar _ | JCEassert _ | JCEreturn_void as enode -> enode
     | JCEcontract(req,dec,vi_result,behs,_e) ->
@@ -1074,6 +1086,8 @@ let replace_sub_expr e el =
 	JCEapp { call with jc_call_args = el }
     | JCEoffset(off,_e,st) ->
 	let e1 = as1 el in JCEoffset(off,e1,st)
+    | JCEfresh(_e) ->
+	let e1 = as1 el in JCEfresh(e1)
     | JCEaddress(absolute,_e) ->
 	let e1 = as1 el in JCEaddress(absolute,e1)
     | JCEbase_block(_e) ->
@@ -1149,6 +1163,7 @@ module ExprAst = struct
       | JCEreal_cast(e, _)
       | JCEoffset(_, e, _)
       | JCEaddress(_,e)
+      | JCEfresh(e)
       | JCEbase_block(e)
       | JCEalloc(e, _)
       | JCEfree e
@@ -1204,6 +1219,7 @@ let fold_sub_expr_and_term_and_assertion
     | JCEoffset(_,e1,_)
     | JCEaddress(_,e1)
     | JCEbase_block(e1)
+    | JCEfresh(e1)
     | JCEcast(e1,_)
     | JCEbitwise_cast(e1,_) 
     | JCErange_cast(e1,_) 
@@ -1306,6 +1322,7 @@ module NExprAst = struct
       | JCNEcast(e, _)
       | JCNEoffset(_, e)
       | JCNEaddress(_,e)
+      | JCNEfresh(e)
       | JCNEbase_block(e)
       | JCNEalloc(e, _)
       | JCNEfree e
