@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -70,8 +70,10 @@ let generic full f targets =
        out "WHYEXEC ?= why@\n";
        out "GWHYEXEC ?= gwhy-bin@\n";
        out "WHYLIB ?= %s@\n@\n" (String.escaped Jc_options.libdir);
-       out "WHY=WHYLIB=$(WHYLIB) $(WHYEXEC) $(WHYOPT) %s -explain -locs %s.loc@\n@\n" (Jc_options.why_opt) f;
-       out "GWHY=WHYLIB=$(WHYLIB) $(GWHYEXEC) $(WHYOPT) %s -explain -locs %s.loc@\n@\n" (Jc_options.why_opt) f;
+       out "USERWHYTWOOPT=%s@\n"  (Jc_options.why_opt);
+       out "USERWHYTHREEOPT=%s@\n"  (Jc_options.why3_opt);
+       out "WHY=WHYLIB=$(WHYLIB) $(WHYEXEC) $(WHYOPT) $(USERWHYTWOOPT) -explain -locs %s.loc@\n@\n" f;
+       out "GWHY=WHYLIB=$(WHYLIB) $(GWHYEXEC) $(WHYOPT) $(USERWHYTWOOPT) -explain -locs %s.loc@\n@\n"  f;
        out "JESSIELIBFILES ?=";
        List.iter (fun s ->
 		    out " %s"
@@ -79,8 +81,7 @@ let generic full f targets =
 					 (Filename.concat "why" s))))
 	 (Jc_options.get_libfiles ());
        out "@\n";
-       out "JESSIE3LIB ?=%s@\n@\n" (String.escaped (Filename.concat "$(WHYLIB)"
-					 "why3"));
+       out "JESSIE3CONF ?= $(WHYLIB)/why3/why3.conf@\n@\n";
        out "COQDEP = coqdep@\n@\n";
 
        out ".PHONY: all coq pvs simplify vampire cvcl harvey smtlib zenon@\n@\n";
@@ -216,19 +217,17 @@ let generic full f targets =
        out "\t@@echo 'why -why3 [...] why/$*.why' \
             && $(WHY) $(JESSIELIBFILES) why/$*.why@\n";
 
-       out "why3ide: why/%s@\n" why3_target;
-       out "\t@@echo 'why3ide [...] $<' \
-            && why3ide $<@\n@\n";
-
        let why3ml_target =
 	 (match targets with f::_ -> f^".mlw" | [] -> "")
        in
        out "why3ml: %s@\n" why3ml_target;
-       out "\t@@echo 'why3ml [...] $<' \
-            && why3ide -I $(JESSIE3LIB) $<@\n@\n";
-(*
-       out "\twhy3ide -I $(JESSIE3LIB) $<@\n@\n";
-*)
+       out "\t why3 $(USERWHYTHREEOPT) --extra-config $(JESSIE3CONF) $<@\n@\n";
+
+       out "why3ide: %s@\n" why3ml_target;
+       out "\t why3ide $(USERWHYTHREEOPT) --extra-config $(JESSIE3CONF) $<@\n@\n";
+
+       out "why3replay: %s@\n" why3ml_target;
+       out "\t why3replayer $(USERWHYTHREEOPT) --extra-config $(JESSIE3CONF) $<@\n@\n";
 
        out "-include %s.depend@\n@\n" f;
        out "depend: %a@\n" (print_files coq_v) targets;
