@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -68,12 +68,12 @@ let return_label = new identifier name_for_return_label
 let label_to_exception = Hashtbl.create 17
 
 let goto_exception_for_label lab =
-  (* CIL defines a label "return_label" to go to before returning. 
+  (* CIL defines a label "return_label" to go to before returning.
      It is recognized here so that static analysis recognizes these returns
      and avoids merging all returns together. *)
   if lab = "return_label" then return_label else
     try
-      Hashtbl.find label_to_exception lab 
+      Hashtbl.find label_to_exception lab
     with Not_found ->
       let excname = Jc_envset.get_unique_name ("Goto_" ^ lab) in
       let exc = new identifier excname in
@@ -88,7 +88,7 @@ let goto_exception_for_label lab =
 (** Transform switch *)
 let normalize_switch pos e caselist =
   (* Give a temporary name to the switch expression, so that modifying
-   * a variable on which this expression depends does not interfere 
+   * a variable on which this expression depends does not interfere
    * with the control-flow, when its value is tested.
    *)
   let epos = e#pos in
@@ -96,21 +96,21 @@ let normalize_switch pos e caselist =
   let tmpvar = mkvar ~pos:epos ~name:tmpname () in
   let has_default c = List.exists (fun c -> c = None) c in
   (* Test for case considered *)
-  let test_one_case ~(neg:bool) c = 
+  let test_one_case ~(neg:bool) c =
     let op = if neg then `Bneq else `Beq in
     mkbinary ~pos:epos ~expr1:tmpvar ~op ~expr2:c ()
   in
   (* Collect negative tests for [default] case *)
-  let all_neg_cases () = 
-    let collect_neg_case c = 
+  let all_neg_cases () =
+    let collect_neg_case c =
       List.fold_right (fun c l -> match c with
 			 | Some c -> test_one_case ~neg:true c :: l
 			 | None -> l) c []
     in
-    fst (List.fold_left (fun (l,after_default) (c,_)  -> 
+    fst (List.fold_left (fun (l,after_default) (c,_)  ->
 			   if after_default then
 			     collect_neg_case c @ l,after_default
-			   else		
+			   else
 			     l,has_default c
 			) ([],false) caselist)
   in
@@ -118,22 +118,22 @@ let normalize_switch pos e caselist =
     | Some c -> test_one_case ~neg:false c
     | None -> mkand ~pos:epos ~list:(all_neg_cases ()) ()
   in
-  let test_case_or_default c = 
+  let test_case_or_default c =
     mkor ~pos:epos ~list:(List.map test_one_case_or_default c) ()
   in
-  let rec cannot_fall_trough e = 
+  let rec cannot_fall_trough e =
     match e#node with
-      | JCPEblock [] -> 
+      | JCPEblock [] ->
 	  false
-      | JCPEblock elist -> 
+      | JCPEblock elist ->
 	  cannot_fall_trough (List.hd (List.rev elist))
-      | JCPEthrow _ | JCPEreturn _ | JCPEwhile _ | JCPEfor _ -> 
+      | JCPEthrow _ | JCPEreturn _ | JCPEwhile _ | JCPEfor _ ->
 	  true
       | JCPEif(_,te,fe) ->
 	  cannot_fall_trough te && cannot_fall_trough fe
       | _ -> false
   in
-  let rec fold_case (previous_c,acc) = 
+  let rec fold_case (previous_c,acc) =
     function [] -> List.rev acc | (c,e) :: next_cases ->
       (* No need to test on previous values if default present *)
       let current_c = if has_default c then c else previous_c @ c in
@@ -147,7 +147,7 @@ let normalize_switch pos e caselist =
       else
 	let ife = mkif ~pos:epos ~condition:teste ~expr_then:e () in
 	fold_case (current_c, ife :: acc) next_cases
-  and start_fold_case caselist = 
+  and start_fold_case caselist =
     let iflist = fold_case ([],[]) caselist in
     mkblock ~pos ~exprs:iflist ()
   in
@@ -171,9 +171,9 @@ let normalize_while pos test behs var body =
   in
   mktry ~pos
     ~expr:
-    (mkwhile ~pos ~behaviors:behs ?variant:var 
+    (mkwhile ~pos ~behaviors:behs ?variant:var
        ~body:
-       (mktry ~pos 
+       (mktry ~pos
           ~expr:
           (mkblock ~pos ~exprs:[body; mkthrow ~pos ~exn:loop_continue ()] ())
 	  ~catches: [mkcatch ~pos ~name:(tmp_var_name()) ~exn:loop_continue ()]
@@ -186,15 +186,15 @@ let normalize_while pos test behs var body =
 (** Transform for-loop *)
 let normalize_for pos inits test updates behs var body =
   mkblock ~pos
-    ~exprs:(inits 
+    ~exprs:(inits
 	    @ [mktry ~pos
                  ~expr:
-		 (mkwhile ~pos ~behaviors:behs ?variant:var 
+		 (mkwhile ~pos ~behaviors:behs ?variant:var
                     ~body:
-		    (mktry ~pos 
+		    (mktry ~pos
                        ~expr:
 		       (mkblock ~pos ~exprs:[
-			  mkif ~pos ~condition:test ~expr_then:body 
+			  mkif ~pos ~condition:test ~expr_then:body
                             ~expr_else:(mkthrow ~pos ~exn:loop_exit ()) ();
 			  mkthrow ~pos ~exn:loop_continue ()] ())
                        ~catches:
@@ -206,19 +206,19 @@ let normalize_for pos inits test updates behs var body =
                  ()
 	      ])
     ()
-    
+
 let duplicable =
-  Jc_iterators.IPExpr.fold_left 
+  Jc_iterators.IPExpr.fold_left
     (fun acc e -> acc && match e#node with
-       | JCPEconst _ | JCPEvar _ | JCPErange _ | JCPEderef _
+       | JCPEconst _ | JCPEvar _ | JCPErange _ | JCPEderef _ | JCPEfresh _
        | JCPEunary _ | JCPEoffset _ | JCPEaddress _ | JCPEold _ | JCPEat _
        | JCPEbinary _ | JCPEcast _ | JCPEsubtype _ | JCPEbase_block _ ->
 	   true
-       | JCPEassert _ | JCPEthrow _ | JCPEreturn _ | JCPEeqtype _  
+       | JCPEassert _ | JCPEthrow _ | JCPEreturn _ | JCPEeqtype _
        | JCPEbreak _ | JCPEcontinue _  | JCPEgoto _  | JCPEdecl _
-       | JCPElabel _ | JCPEinstanceof _ | JCPEalloc _ 
-       | JCPEfree _ | JCPElet _ | JCPEpack _ | JCPEunpack _ 
-       | JCPEquantifier _ | JCPEmutable _ | JCPEassign _ 
+       | JCPElabel _ | JCPEinstanceof _ | JCPEalloc _
+       | JCPEfree _ | JCPElet _ | JCPEpack _ | JCPEunpack _
+       | JCPEquantifier _ | JCPEmutable _ | JCPEassign _
        | JCPEassign_op _ | JCPEif _ | JCPEwhile _ | JCPEblock _
        | JCPEapp _ | JCPEtry _ | JCPEmatch _ | JCPEfor _
        | JCPEswitch _ | JCPEcontract _ ->
@@ -254,7 +254,7 @@ let normalize_assign_op pos e1 op e2 =
 (** Transform unary increment and decrement *)
 let normalize_pmunary pos op e =
   let op_of_incdec = function
-    | `Uprefix_inc | `Upostfix_inc -> `Badd 
+    | `Uprefix_inc | `Upostfix_inc -> `Badd
     | `Uprefix_dec | `Upostfix_dec -> `Bsub
     | `Uplus -> assert false
   in
@@ -297,7 +297,7 @@ let normalize_pmunary pos op e =
       | `Uplus -> assert false
   in
   match op with `Uplus -> e | _ ->
-    if duplicable e then on_duplicable e else 
+    if duplicable e then on_duplicable e else
       match e#node with
 	| JCPEderef(e1,f) ->
 	    let tmpname = tmp_var_name () in
@@ -307,7 +307,7 @@ let normalize_pmunary pos op e =
 	| _ -> Jc_options.jc_error pos "Not an lvalue in assignment"
 
 (** Transform local variable declarations *)
-let normalize_locvardecl pos elist = 
+let normalize_locvardecl pos elist =
   mkblock ~pos
     ~exprs:
     (List.fold_right
@@ -319,12 +319,12 @@ let normalize_locvardecl pos elist =
 	    | JCPElabel(lab, e1) ->
 		(* the scope of lab is indeed extended to the end of that block, so that e.g.
 		   { (L: e1); ... ; assert ... \at(x,L) ... }
-		   
+
 		   is valid
 		*)
 		[mklabel
                   ~pos:e#pos
-                  ~label:lab 
+                  ~label:lab
 		  (* CAUTION ! shouldn't we recurse normalize on e1 ??? *)
                   ~expr:(mkblock ~pos ~exprs:(e1::acc) ())
 		  ()]
@@ -356,7 +356,7 @@ let normalize_postaction pos elist =
     | `Upostfix_inc -> `Uprefix_inc
     | `Upostfix_dec -> `Uprefix_dec
   in
-  mkblock ~pos 
+  mkblock ~pos
     ~exprs:
     (match List.rev elist with [] -> elist | last::elist' ->
        (* Only transform into pre increment/decrement those post increment/
@@ -364,8 +364,8 @@ let normalize_postaction pos elist =
 	* but the last one.
 	*)
        (List.fold_left (fun acc e -> match e#node with
-			  | JCPEunary(#post_unary_op as op,e') -> 
-			      new pexpr_with 
+			  | JCPEunary(#post_unary_op as op,e') ->
+			      new pexpr_with
 				~node:(JCPEunary(pre_of_post op,e')) e
 			      :: acc
 			  | _ -> e :: acc
@@ -374,19 +374,19 @@ let normalize_postaction pos elist =
     ()
 
 (** Apply normalizations recursively *)
-let normalize = 
-  Jc_iterators.map_pexpr 
+let normalize =
+  Jc_iterators.map_pexpr
     ~before:(fun e -> match e#node with
 	       | JCPEblock elist ->
 		   normalize_postaction e#pos elist
 	       | _ -> e
 	    )
     ~after:(fun e -> match e#node with
-	      | JCPEassign_op(e1,op,e2) -> 
+	      | JCPEassign_op(e1,op,e2) ->
 		  normalize_assign_op e#pos e1 op e2
-	      | JCPEunary(#pm_unary_op as op,e') -> 
+	      | JCPEunary(#pm_unary_op as op,e') ->
 		  normalize_pmunary e#pos op e'
-	      | JCPEswitch(e',caselist) -> 
+	      | JCPEswitch(e',caselist) ->
 		  normalize_switch e#pos e' caselist
 	      | JCPEwhile(test,inv,var,body) ->
 		  normalize_while e#pos test inv var body
@@ -416,9 +416,9 @@ type label_tree =
   | LabelBlock of label_tree list
 
 let rec printf_label_tree fmt lt =
-  match lt with 
+  match lt with
     | LabelItem s -> fprintf fmt "%s" s
-    | LabelBlock l -> 
+    | LabelBlock l ->
 	fprintf fmt "{ %a }" (Pp.print_list Pp.space printf_label_tree ) l
 
 let rec in_label_tree lab = function
@@ -427,7 +427,7 @@ let rec in_label_tree lab = function
 
 and in_label_tree_list lab = function
   | [] -> false
-  | h::r -> 
+  | h::r ->
       in_label_tree lab h || in_label_tree_list lab r
 
 let rec in_label_upper_tree_list lab = function
@@ -436,9 +436,9 @@ let rec in_label_upper_tree_list lab = function
   | _ :: r -> in_label_upper_tree_list lab r
 
 let build_label_tree e : label_tree list =
-  (* [acc] is the tree of labels for the list of statements that follow 
+  (* [acc] is the tree of labels for the list of statements that follow
      the current one, in the same block.
-     [fwdacc] is the tree of labels for all the statements that follow 
+     [fwdacc] is the tree of labels for all the statements that follow
      the current one to the end of the function. It is used to identify
      unused labels.
   *)
@@ -449,7 +449,7 @@ let build_label_tree e : label_tree list =
 	     not be considered in generated try-catch. *)
 	  if in_label_upper_tree_list lab fwdacc then
 	    Hashtbl.add label_used lab ()
-	  else 
+	  else
 	    Jc_options.jc_error e#pos "unsupported goto (backward or to some inner block)";
 	  acc,fwdacc
       | JCPElabel (lab, e) ->
@@ -460,14 +460,14 @@ let build_label_tree e : label_tree list =
 	  (LabelBlock l) :: acc, fwdl
       | _ ->
 	  let elist = Jc_iterators.IPExpr.subs e in
-	  LabelBlock 
+	  LabelBlock
 	    (List.map (fun e -> LabelBlock(fst (build_bwd e ([],fwdacc)))) elist)
 	  :: acc, fwdacc
   in
   fst (build_bwd e ([],[]))
 
 let goto_block pos el =
-  let rec label_block el = 
+  let rec label_block el =
     match el with [] -> [],[] | e1::r ->
       let elr,labelr = label_block r in
       match e1#node with
@@ -485,7 +485,7 @@ let goto_block pos el =
 		    [mktry ~pos
                        ~expr:(mkblock ~pos ~exprs:(acc@[throw]) ())
 		       ~catches:
-                       [mkcatch ~pos ~name:(tmp_var_name()) ~exn:id 
+                       [mkcatch ~pos ~name:(tmp_var_name()) ~exn:id
 			  ~body:(mkblock ~pos ~exprs:el ()) ()]
                        ()
 		    ]
@@ -496,7 +496,7 @@ let goto_block pos el =
 let rec goto e lz =
   let pos = e#pos in
   let enode,lz2 = match e#node with
-    | JCPEgoto lab -> 
+    | JCPEgoto lab ->
 	let id = goto_exception_for_label lab in
 	JCPEthrow(id, mkvoid ()), lz
     | JCPElabel (lab,e1) ->
@@ -508,13 +508,13 @@ let rec goto e lz =
 	in
 	let e2, lz2 = goto e1 lz1 in
 	JCPElabel(lab,e2), lz2
-    | JCPEblock el -> 
+    | JCPEblock el ->
 	let lz1,lz2 = match lz with
 	  | LabelBlock b1::after ->
 	      b1@after,after
 	  | _ -> assert false
 	in
-	let el,_ = 
+	let el,_ =
 	  List.fold_left (fun (acc,lz1) e1 ->
 			    let e2,lz2 = goto e1 lz1 in e2::acc,lz2
 			 ) ([],lz1) el
@@ -554,21 +554,22 @@ let rec expr e =
     | JCPEassign_op _ -> assert false
     | JCPEinstanceof(e,id) -> JCNEinstanceof(expr e,id)
     | JCPEcast(e,id) -> JCNEcast(expr e,id)
-    | JCPEquantifier(q,ty,idlist,trigs,e) -> 
+    | JCPEquantifier(q,ty,idlist,trigs,e) ->
         JCNEquantifier(q,ty,idlist,List.map (List.map expr) trigs,expr e)
     | JCPEold e -> JCNEold(expr e)
     | JCPEat(e,lab) -> JCNEat(expr e,lab)
     | JCPEoffset(off,e) -> JCNEoffset(off,expr e)
+    | JCPEfresh e -> JCNEfresh(expr e)
     | JCPEaddress(absolute,e) -> JCNEaddress(absolute,expr e)
     | JCPEbase_block(e) -> JCNEbase_block(expr e)
     | JCPEif(e1,e2,e3) -> JCNEif(expr e1,expr e2,expr e3)
-    | JCPElet(tyopt,id,e1,e2) -> 
+    | JCPElet(tyopt,id,e1,e2) ->
 	JCNElet(tyopt,id,Option_misc.map expr e1,expr e2)
     | JCPEdecl _ ->
 	assert false
 	(* (ty,name,initopt) ->  *)
 (* 	JCNElet(Some ty,name,Option_misc.map expr initopt,expr (mkvoid())) *)
-    | JCPErange(e1opt,e2opt) -> 
+    | JCPErange(e1opt,e2opt) ->
 	JCNErange(Option_misc.map expr e1opt,Option_misc.map expr e2opt)
     | JCPEalloc(e,id) -> JCNEalloc(expr e,id)
     | JCPEfree e -> JCNEfree(expr e)
@@ -576,12 +577,12 @@ let rec expr e =
     | JCPEeqtype(tag1,tag2) -> JCNEeqtype(tag_ tag1,tag_ tag2)
     | JCPEsubtype(tag1,tag2) -> JCNEsubtype(tag_ tag1,tag_ tag2)
     | JCPEmatch(e,pelist) ->
-	JCNEmatch(expr e,List.map (fun (pat,e) -> (pat,expr e)) pelist)  
+	JCNEmatch(expr e,List.map (fun (pat,e) -> (pat,expr e)) pelist)
     | JCPEblock elist -> JCNEblock(List.map expr elist)
     | JCPEassert(behav,asrt,e) -> JCNEassert(behav,asrt,expr e)
-    | JCPEcontract(req,dec,behs,e) -> 
+    | JCPEcontract(req,dec,behs,e) ->
 	JCNEcontract(Option_misc.map expr req,
-		     Option_misc.map 
+		     Option_misc.map
 		       (fun (t,r) -> (expr t,r)) dec,
 		     List.map behavior behs,
 		     expr e)
@@ -608,18 +609,19 @@ let rec expr e =
   in
   new nexpr ~pos:e#pos enode
 
-and tag_ tag = 
+and tag_ tag =
   let tagnode = match tag#node with
     | JCPTtypeof e -> JCPTtypeof (expr e)
     | JCPTtag _ | JCPTbottom as tagnode -> tagnode
   in
   new ptag ~pos:tag#pos tagnode
 
-and behavior (pos,id,idopt,e1opt,e2opt,asslist,e3) =
+and behavior (pos,id,idopt,e1opt,e2opt,asslist,alloclist,e3) =
   (pos,id,idopt,
    Option_misc.map expr e1opt,
    Option_misc.map expr e2opt,
    Option_misc.map (fun (pos,elist) -> pos,List.map expr elist) asslist,
+   Option_misc.map (fun (pos,elist) -> pos,List.map expr elist) alloclist,
    expr e3)
 
 and loopbehavior (ids,inv,asslist) =
@@ -642,7 +644,7 @@ let clause = function
   | JCCdecreases(e,r) -> JCCdecreases(expr e,r)
   | JCCbehavior b -> JCCbehavior(behavior b)
 
-    
+
 (** From parsed reads-or-expr to normalized reads-or-expr *)
 let reads_or_expr = function
   | JCnone -> JCnone
@@ -652,9 +654,9 @@ let reads_or_expr = function
   | JCaxiomatic l -> JCaxiomatic(List.map (fun (id,e) -> (id, expr e)) l)
 *)
   | JCinductive l -> JCinductive(List.map (fun (id,labels,e) -> (id, labels, expr e)) l)
-    
+
 (** From parsed declaration to normalized declaration *)
-let rec decl d = 
+let rec decl d =
   let dnode = match d#node with
     | JCDfun(ty,id,params,clauses,body) ->
 	JCDfun(ty,id,params,List.map clause clauses,Option_misc.map expr body)
@@ -665,7 +667,7 @@ let rec decl d =
     | JCDunion_type(id,discr,tags) ->
 	JCDunion_type(id,discr,tags)
     | JCDtag (id, params, extends, fields, invs) ->
-	JCDtag (id, params, extends, fields, 
+	JCDtag (id, params, extends, fields,
 		List.map (fun (id,name,e) -> id,name,expr e) invs)
     | JCDvar(ty,id,init) ->
 	JCDvar(ty,id,Option_misc.map expr init)
@@ -688,11 +690,13 @@ let rec decl d =
     | JCDseparation_policy _
     | JCDinvariant_policy _
     | JCDpragma_gen_sep _
-    | JCDpragma_gen_frame _ as e -> e
+    | JCDpragma_gen_frame _
+    | JCDpragma_gen_sub _
+    | JCDpragma_gen_same _ as e -> e
 
 
   in new decl ~pos:d#pos dnode
-	  
+
 let decls dlist =
   let unit_type = new ptype (JCPTnative Tunit) in
   [
@@ -707,7 +711,7 @@ let decls dlist =
   @ List.map decl dlist
 
 (*
-  Local Variables: 
+  Local Variables:
   compile-command: "LC_ALL=C make -C .. bin/jessie.byte"
-  End: 
+  End:
 *)
