@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -81,15 +81,17 @@ let type_only = ref false
 let user_annot_only = ref false
 let print_graph = ref false
 let why_opt = ref ""
+let why3_opt = ref ""
 
 let verify_all_offsets = ref false
 let verify_invariants_only = ref false
 let verify = ref []
-let behavior = ref ""
+let behavior = ref []
 
 let why3_backend = ref false
 
 let add_why_opt s = why_opt := !why_opt ^ " " ^ s
+let add_why3_opt s = why3_opt := !why3_opt ^ " " ^ s
 
 let annotation_sem = ref AnnotNone
 let ai_domain = ref AbsNone
@@ -115,7 +117,7 @@ let pos_table = Hashtbl.create 97
 
 let version () =
   Printf.printf "This is Jessie version %s, compiled on %s
-Copyright (c) 2006-2011 - CNRS/INRIA/Univ Paris 11, team ProVal
+Copyright (c) 2006-2014 - CNRS/INRIA/Univ Paris-Sud
 This is free software with ABSOLUTELY NO WARRANTY (use option -warranty)
 " Version.version Version.date;
   exit 0
@@ -136,14 +138,16 @@ let _ =
           "  debugging mode";
 	"-locs", Arg.String (fun f -> pos_files := f :: !pos_files),
 	  "  <f> reads source locations from file f" ;
-        "-behavior", Arg.String (fun s -> behavior := s),
-	  "  verify only specified behavior (safety, default or user-defined behavior)";
+        "-behavior", Arg.String (fun s -> behavior := s::!behavior),
+	  "  verify only specified behavior (safety, variant, default or user-defined behavior)";
 
         "-why3ml", Arg.Set why3_backend,
           "  (experimental) produce a program in why3ml syntax" ;
 
         "-why-opt", Arg.String add_why_opt,
 	  "  <why options>  passes options to Why";
+        "-why3-opt", Arg.String add_why3_opt,
+	  "  <why options>  passes options to Why3";
 	"-v", Arg.Set verbose,
           "  verbose mode";
 	"-q", Arg.Clear verbose,
@@ -205,6 +209,7 @@ let verbose = !verbose
 let werror = !werror
 let why3_backend = !why3_backend
 let why_opt = !why_opt
+let why3_opt = !why3_opt
 let inv_sem = inv_sem
 let separation_sem = separation_sem
 let trust_ai = !trust_ai
@@ -221,7 +226,7 @@ let gen_frame_rule_with_ft = !gen_frame_rule_with_ft
 let () = if gen_frame_rule_with_ft then libfiles := "mybag.why"::!libfiles
 
 let verify_behavior s =
-  behavior = "" || behavior = s
+  behavior = [] || List.mem s behavior
 
 let set_int_model im =
   if im = IMmodulo && trust_ai then
