@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -58,7 +58,7 @@ let fprintf_constant form e =
     | Prim_real(f) -> fprintf form "%s" f
     | Prim_bool(b) ->
         if !why3syntax then
-          fprintf form "%s" (if b then "True" else "False")
+          fprintf form "%s" (if b then "Bool.True" else "Bool.False")
         else
           fprintf form "%b" b
 (*
@@ -96,7 +96,9 @@ let rec match_term acc t1 t2 =
 let why3id s =
   match s.[0] with
   | 'A'..'Z' -> "_" ^ s
-  | _ -> s
+  | _ ->
+      if Why3_kw.is_why3_keyword s then s ^ "_why3" else s
+
 
 let why3constr s =
   match s.[0] with
@@ -109,6 +111,13 @@ let why3id_if s =
 
 let why3ident s =
   match s with
+        (* booleans *)
+    | "not" -> "not"
+    | "bool_and" -> "Bool.andb"
+    | "bool_or" -> "Bool.orb"
+    | "bool_xor" -> "Bool.xorb"
+    | "bool_not" -> "Bool.notb"
+        (* integers *)
     | "le_int" -> "Int.(<=)"
     | "le_int_" -> "Int.(<=)"
     | "le_int_bool" -> "Int.(<=)"
@@ -152,31 +161,48 @@ let why3ident s =
     | "truncate_real_to_int" -> "Truncate.truncate"
     | "real_min" -> "RealMinMax.min"
     | "real_max" -> "RealMinMax.max"
+    | "abs_int" -> "AbsInt.abs"
     | "abs_real" -> "AbsReal.abs"
     | "sqrt_real" -> "Square.sqrt"
-    | "pow_real" -> "Power.pow"
+    | "pow_int" -> "Power.power"
+    | "pow_real" -> "PowerReal.pow"
+    | "pow_real_int" -> "PowerInt.power"
+    | "exp" -> "ExpLog.exp"
+    | "log" -> "ExpLog.log"
     | "cos" -> "Trigonometry.cos"
     | "sin" -> "Trigonometry.sin"
     | "tan" -> "Trigonometry.tan"
     | "atan" -> "Trigonometry.atan"
         (* floats *)
     | "nearest_even" -> "Rounding.NearestTiesToEven"
+    | "down" -> "Rounding.Down"
     | "single_value" -> "Single.value"
     | "single_exact" -> "Single.exact"
     | "single_round_error" -> "Single.round_error"
     | "round_single" -> "Single.round"
+    | "round_single_logic" -> "Single.round_logic"
+    | "no_overflow_single" -> "Single.no_overflow"
     | "double_value" -> "Double.value"
     | "double_exact" -> "Double.exact"
     | "double_round_error" -> "Double.round_error"
     | "round_double" -> "Double.round"
+    | "round_double_logic" -> "Double.round_logic"
+    | "no_overflow_double" -> "Double.no_overflow"
         (* floats full *)
-    | "le_double_full" -> "DoubleFull.le_full"
-    | "lt_double_full" -> "DoubleFull.lt_full"
-    | "ge_double_full" -> "DoubleFull.ge_full"
-    | "gt_double_full" -> "DoubleFull.gt_full"
-    | "eq_double_full" -> "DoubleFull.eq_full"
-    | "ne_double_full" -> "DoubleFull.ne_full"
-    | "double_is_NaN" -> "DoubleFull.is_NaN"
+    | "le_double_full" -> "Double.le"
+    | "lt_double_full" -> "Double.lt"
+    | "ge_double_full" -> "Double.ge"
+    | "gt_double_full" -> "Double.gt"
+    | "eq_double_full" -> "Double.eq"
+    | "ne_double_full" -> "Double.ne"
+    | "double_is_finite" -> "Double.is_finite"
+    | "double_is_infinite" -> "Double.is_infinite"
+    | "double_is_plus_infinity" -> "Double.is_plus_infinity"
+    | "double_is_minus_infinity" -> "Double.is_minus_infinity"
+    | "double_is_NaN" -> "Double.is_NaN"
+    | "double_sign" -> "Double.sign"
+    | "Positive" -> "SpecialValues.Pos"
+    | "Negative" -> "SpecialValues.Neg"
     | _ -> why3id s
 
 let why3ident_if s =
@@ -204,8 +230,12 @@ let why3_FromInt = ref false
 let why3_Truncate = ref false
 let why3_Square = ref false
 let why3_Power = ref false
+let why3_PowerInt = ref false
+let why3_PowerReal = ref false
 let why3_RealMinMax = ref false
+let why3_AbsInt = ref false
 let why3_AbsReal = ref false
+let why3_ExpLog = ref false
 let why3_Trigonometry = ref false
 
 let compute_why3_dependencies f =
@@ -220,12 +250,16 @@ let compute_why3_dependencies f =
     | "mul_real"
     | "div_real" -> why3_reals := true
     | "sqrt_real" -> why3_Square := true
-    | "pow_real" -> why3_Power := true
+    | "pow_int" -> why3_Power := true
+    | "pow_real_int" -> why3_PowerInt := true
+    | "pow_real" -> why3_PowerReal := true
     | "real_of_int" -> why3_FromInt := true
     | "truncate_real_to_int" -> why3_Truncate := true
-    | "real_min" 
+    | "real_min"
     | "real_max" -> why3_RealMinMax := true
+    | "abs_int" -> why3_AbsInt := true
     | "abs_real" -> why3_AbsReal := true
+    | "exp" | "log" -> why3_ExpLog := true
     | "cos"
     | "sin"
     | "tan"
@@ -397,7 +431,7 @@ let why3loc ~prog fmt lab =
   with
       Not_found ->
         if prog then
-          fprintf fmt "'%s:" (why3constr lab)
+          fprintf fmt "'%s: " (why3constr lab)
         else
           fprintf fmt "\"%s\"" lab
 
@@ -407,63 +441,9 @@ let is_why3_local id = Hashtbl.mem why3_locals id
 
 let add_why3_local id = Hashtbl.add why3_locals id ()
 
-let remove_why3_local id = Hashtbl.remove why3_locals id 
+let () = add_why3_local "result"
 
-let rec fprintf_term form t =
-  match t with
-  | LConst(c) -> fprintf_constant form c
-  | LApp("eq_pointer",[t1;t2]) ->
-      fprintf form "@[(%a=%a)@]"
-	fprintf_term t1
-	fprintf_term t2
-  | LApp("ne_pointer",[t1;t2]) ->
-      fprintf form "@[(%a<>%a)@]"
-	fprintf_term t1
-	fprintf_term t2
-  | LApp(id,t::tl) ->
-      if !why3syntax then
-        begin
-          fprintf form "@[(%s@ %a" (why3ident id) fprintf_term t;
-          List.iter (fun t -> fprintf form "@ %a" fprintf_term t) tl;
-          fprintf form ")@]"
-        end
-      else
-        begin
-          fprintf form "@[%s(%a" id fprintf_term t;
-          List.iter (fun t -> fprintf form ",@ %a" fprintf_term t) tl;
-          fprintf form ")@]"
-        end
-  | LApp(id,[]) ->
-      fprintf form "%s" (why3ident_if id)
-  | LVar id ->
-      fprintf form "%s" (why3ident_if id)
-  | LDeref id ->
-      if !why3syntax then
-        if is_why3_local id then
-          fprintf form "%s" (why3ident_if id)
-        else
-          fprintf form "!%s" (why3ident id)
-      else
-        fprintf form "%s" id
-  | LDerefAtLabel(id,l) ->
-      if !why3syntax then
-        if l="" then
-          fprintf form "(old !%s)" (why3ident id)
-        else
-          fprintf form "(at !%s '%s)" (why3ident id) (why3constr l)
-      else
-        fprintf form "%s@@%s" id l
-  | Tnamed(lab,t) ->
-      if !why3syntax then
-        fprintf form "(%a %a)" (why3loc ~prog:false) lab fprintf_term t
-      else
-        fprintf form "(%s : %a)" lab fprintf_term t
-  | TIf(t1,t2,t3) ->
-      fprintf form "@[<hov 1>(if %a@ then %a@ else %a)@]"
-	fprintf_term t1 fprintf_term t2 fprintf_term t3
-  | TLet(v,t1,t2) ->
-      fprintf form "@[<hov 1>(let %s@ = %a@ in %a)@]" v
-	fprintf_term t1 fprintf_term t2
+let remove_why3_local id = Hashtbl.remove why3_locals id
 
 type logic_type =
     { logic_type_name : string;
@@ -604,6 +584,7 @@ let logic_type_name t =
   if !why3syntax then
     match t.logic_type_name with
       | "unit" -> "()"
+      | "bool" -> "Bool.bool"
       | s -> why3ident s
   else
     t.logic_type_name
@@ -626,7 +607,90 @@ let rec fprintf_logic_type form t =
 	    (print_list simple_comma fprintf_logic_type) l
 	    t.logic_type_name
 
-let rec fprintf_assertion form a =
+let rec bool_to_prop t =
+  match t with
+    | LConst (Prim_bool true) -> LTrue
+    | LConst (Prim_bool false) -> LFalse
+    | LApp("bool_not",[t1]) -> LNot(bool_to_prop t1)
+    | LApp("bool_and",[t1;t2]) -> LAnd(bool_to_prop t1,bool_to_prop t2)
+    | LApp("bool_or",[t1;t2]) -> LOr(bool_to_prop t1,bool_to_prop t2)
+    | LApp("lt_int_bool",[t1;t2]) -> LPred("lt_int",[t1;t2])
+    | LApp("le_int_bool",[t1;t2]) -> LPred("le_int",[t1;t2])
+    | LApp("gt_int_bool",[t1;t2]) -> LPred("gt_int",[t1;t2])
+    | LApp("ge_int_bool",[t1;t2]) -> LPred("ge_int",[t1;t2])
+    | LApp("lt_real_bool",[t1;t2]) -> LPred("lt_real",[t1;t2])
+    | LApp("le_real_bool",[t1;t2]) -> LPred("le_real",[t1;t2])
+    | LApp("gt_real_bool",[t1;t2]) -> LPred("gt_real",[t1;t2])
+    | LApp("ge_real_bool",[t1;t2]) -> LPred("ge_real",[t1;t2])
+      (** by default *)
+    | _ -> LPred("eq",[t;LConst(Prim_bool true)])
+
+let is_why3_poly_eq, is_why3_poly_neq =
+  let eqs = ["eq_int"; "eq_bool"; "eq_real"; "eq_int_"; "eq_bool_"; "eq_real_"] in
+  ListLabels.mem ~set:eqs, ListLabels.mem ~set:(List.map ((^) "n") eqs)
+
+let rec fprintf_term form t =
+  match t with
+  | LConst(c) -> fprintf_constant form c
+  | LApp("eq_pointer",[t1;t2]) ->
+      fprintf form "@[(%a=%a)@]"
+	fprintf_term t1
+	fprintf_term t2
+  | LApp("ne_pointer",[t1;t2]) ->
+      fprintf form "@[(%a<>%a)@]"
+	fprintf_term t1
+	fprintf_term t2
+  | LApp(id,t::tl) ->
+      if !why3syntax then
+        begin
+          fprintf form "@[(%s@ %a" (why3ident id) fprintf_term t;
+          List.iter (fun t -> fprintf form "@ %a" fprintf_term t) tl;
+          fprintf form ")@]"
+        end
+      else
+        begin
+          fprintf form "@[%s(%a" id fprintf_term t;
+          List.iter (fun t -> fprintf form ",@ %a" fprintf_term t) tl;
+          fprintf form ")@]"
+        end
+  | LApp(id,[]) ->
+      fprintf form "%s" (why3ident_if id)
+  | LVar id ->
+      fprintf form "%s" (why3ident_if id)
+  | LDeref id ->
+      if !why3syntax then
+        if is_why3_local id then
+          fprintf form "%s" (why3ident_if id)
+        else
+          fprintf form "!%s" (why3ident id)
+      else
+        fprintf form "%s" id
+  | LDerefAtLabel(id,l) ->
+      if !why3syntax then
+        if l="" then
+          fprintf form "(old !%s)" (why3ident id)
+        else
+          fprintf form "(at !%s '%s)" (why3ident id) (why3constr l)
+      else
+        fprintf form "%s@@%s" id l
+  | Tnamed(lab,t) ->
+      if !why3syntax then
+        fprintf form "(%a %a)" (why3loc ~prog:false) lab fprintf_term t
+      else
+        fprintf form "(%s : %a)" lab fprintf_term t
+  | TIf(t1,t2,t3) ->
+    if !why3syntax then
+      let f = bool_to_prop t1 in
+      fprintf form "@[<hov 1>(if %a@ then %a@ else %a)@]"
+        fprintf_assertion f fprintf_term t2 fprintf_term t3
+    else
+      fprintf form "@[<hov 1>(if %a@ then %a@ else %a)@]"
+        fprintf_term t1 fprintf_term t2 fprintf_term t3
+  | TLet(v,t1,t2) ->
+      fprintf form "@[<hov 1>(let %s@ = %a@ in %a)@]" v
+	fprintf_term t1 fprintf_term t2
+
+and fprintf_assertion form a =
   match a with
   | LTrue -> fprintf form "true"
   | LFalse -> fprintf form "false"
@@ -683,7 +747,7 @@ let rec fprintf_assertion form a =
       fprintf form "@[(%a >= %a)@]"
 	fprintf_term t1
 	fprintf_term t2
-  | LPred(id,[t1;t2]) when id = "eq" || !why3syntax && id = "eq_int" ->
+  | LPred(id,[t1;t2]) when id = "eq" || !why3syntax && is_why3_poly_eq id ->
       fprintf form "@[(%a = %a)@]"
 	fprintf_term t1
 	fprintf_term t2
@@ -755,7 +819,7 @@ let rec iter_why_type f t =
 	List.iter f reads;
 	List.iter f writes;
 	iter_assertion f post;
-	List.iter (fun (_,a) -> iter_assertion f a) signals
+	List.iter (fun (id,a) -> f id; iter_assertion f a) signals
 ;;
 
 
@@ -771,12 +835,12 @@ let rec fprintf_type ~need_colon anon form t =
   match t with
     | Prod_type(id,t1,t2) ->
 	  if !why3syntax then
-            let id = if id="" or anon then "_anonymous" else id in
+            let id = if id="" || anon then "_anonymous" else id in
             fprintf form "@[<hov 1>(%s:%a)@ %a@]" (why3ident id)
 	    (fprintf_type ~need_colon:false anon) t1
               (fprintf_type ~need_colon anon) t2
           else
-	    if id="" or anon then
+	    if id="" || anon then
 	      fprintf form "@[<hov 1>%a ->@ %a@]"
 	        (fprintf_type ~need_colon:false anon) t1
                 (fprintf_type ~need_colon anon) t2
@@ -796,17 +860,26 @@ let rec fprintf_type ~need_colon anon form t =
     | Annot_type(p,t,reads,writes,q,signals) ->
 	begin
           if need_colon then fprintf form ": ";
-	  fprintf form "@[@[<hov 2>{ ";
-	  if is_not_true p
-	  then fprintf_assertion form p;
-	  fprintf form "}@]@ %a@ " (fprintf_type ~need_colon:false anon) t;
+          if !why3syntax then
+            begin
+	      fprintf form "%a@ " (fprintf_type ~need_colon:false anon) t;
+	      fprintf form "@[@[<hov 2>requires { %a }@]@ "
+                fprintf_assertion p;
+            end
+          else
+            begin
+	      fprintf form "@[@[<hov 2>{ ";
+	      if is_not_true p
+	      then fprintf_assertion form p;
+	      fprintf form "}@]@ %a@ " (fprintf_type ~need_colon:false anon) t
+            end;
 	  begin
 	    match List.sort compare reads with
 	      | [] -> ()
 	      | r::l as reads ->
                   if !why3syntax then
-		    fprintf form "reads@ %a@ "
-                      (print_list space
+		    fprintf form "reads@ { %a } @ "
+                      (print_list comma
                          (fun form r -> fprintf form "%s" (why3ident r))) reads
                   else
 		  fprintf form "reads %s%a@ " r fprint_comma_string_list l
@@ -816,8 +889,8 @@ let rec fprintf_type ~need_colon anon form t =
 	      | [] -> ()
 	      | r::l as writes ->
                   if !why3syntax then
-		    fprintf form "writes@ %a@ "
-                      (print_list space
+		    fprintf form "writes@ { %a }@ "
+                      (print_list comma
                          (fun form r -> fprintf form "%s" (why3ident r))) writes
                   else
 		    fprintf form "writes %s%a@ " r fprint_comma_string_list l
@@ -825,19 +898,21 @@ let rec fprintf_type ~need_colon anon form t =
 	  begin
 	    match signals with
 	      | [] ->
-		  fprintf form "@[<hov 2>{ %a }@]@]" fprintf_assertion q
+                  if !why3syntax then
+		    fprintf form "@[<hov 2> ensures { %a }@]@]" fprintf_assertion q
+                  else
+		    fprintf form "@[<hov 2>{ %a }@]@]" fprintf_assertion q
 	      | l ->
                   if !why3syntax then
-		    fprintf form
-		      "raises%a@ @[<hov 2>{ %a@ } | %a @]@]"
-		      (print_list comma (fun fmt (e,_r) -> fprintf fmt " %s" e))
-		      l
-		      fprintf_assertion q
-		      (print_list alt
-                         (fun fmt (e,r) ->
-			    fprintf fmt "@[<hov 2>%s ->@ { %a }@]" e
-			      fprintf_assertion r))
-		      l
+		    begin
+                      fprintf form "@[<hov 2> ensures { %a@ }@]@ "
+		        fprintf_assertion q;
+		      List.iter
+                         (fun (e,r) ->
+			    fprintf form "@[<hov 2>raises { %s result ->@ %a }@]" e
+			      fprintf_assertion r)
+		        l
+                    end
                   else
 		    fprintf form
 		      "raises%a@ @[<hov 2>{ %a@ | %a }@]@]"
@@ -883,7 +958,8 @@ type expr_node =
       (int * bool * bool * string) list
   | Let of string * expr * expr
   | Let_ref of string * expr * expr
-  | App of expr * expr
+ (* optionaly give the return type to enforce *)
+  | App of expr * expr * why_type option
   | Raise of string * expr option
   | Try of expr * string * string option * expr
   | Fun of (string * why_type) list *
@@ -897,11 +973,12 @@ type expr_node =
 
 and expr =
     { expr_labels : string list;
+      expr_loc_labels : string list;
       expr_node : expr_node;
     }
 ;;
 
-let mk_expr e = { expr_labels = []; expr_node = e }
+let mk_expr e = { expr_labels = []; expr_loc_labels = []; expr_node = e }
 let mk_var s = mk_expr (Var s)
 let void = mk_expr Void
 
@@ -932,7 +1009,7 @@ let rec iter_expr f e =
         assert false
     | Let(_id,e1,e2) -> iter_expr f e1; iter_expr f e2
     | Let_ref(_id,e1,e2) -> iter_expr f e1; iter_expr f e2
-    | App(e1,e2) -> iter_expr f e1; iter_expr f e2
+    | App(e1,e2,_) -> iter_expr f e1; iter_expr f e2
     | Raise (_, None) -> ()
     | Raise(_id,Some e) -> iter_expr f e
     | Try(e1,_exc,_id,e2) -> iter_expr f e1; iter_expr f e2
@@ -965,7 +1042,7 @@ let fprintf_variant form = function
       else
         fprintf form "variant %a for %s" fprintf_term t r
 
-let rec fprintf_expr_node form e =
+let rec fprintf_expr_node in_app form e =
   match e with
     | Cte(c) -> fprintf_constant form c
     | Var(id) ->
@@ -1024,13 +1101,21 @@ let rec fprintf_expr_node form e =
     | Let_ref(id,e1,e2) ->
         fprintf form "@[<hov 0>(let %s =@ ref %a in@ %a)@]" (why3id_if id)
 	  fprintf_expr e1 fprintf_expr e2
-    | App({expr_node = App({expr_node = Var id},e1)},e2)
-        when !why3syntax && id="eq_int_" ->
+    | App({expr_node = App({expr_node = Var id},e1,_)},e2,_)
+        when !why3syntax && is_why3_poly_eq id ->
 	fprintf form "@[<hov 1>(%a = %a)@]" fprintf_expr e1 fprintf_expr e2
-    | App({expr_node = App({expr_node = Var id},e1)},e2)
-        when !why3syntax && id="neq_int_" ->
+    | App({expr_node = App({expr_node = Var id},e1,_)},e2,_)
+        when !why3syntax && is_why3_poly_neq id ->
 	fprintf form "@[<hov 1>(%a <> %a)@]" fprintf_expr e1 fprintf_expr e2
-    | App(e1,e2) ->
+    | App(e1,e2,ty) when !why3syntax ->
+        if in_app then
+	  fprintf form "@[<hov 1>%a %a@]"
+            (fprintf_expr_gen true) e1 fprintf_expr e2
+        else
+	  fprintf form "@[<hov 1>(%a %a%a)@]"
+            (fprintf_expr_gen true) e1 fprintf_expr e2
+            (Pp.print_option (fprintf_type ~need_colon:true false)) ty
+    | App(e1,e2,_) ->
 	fprintf form "@[<hov 1>(%a %a)@]" fprintf_expr e1 fprintf_expr e2
     | Raise(id,None) ->
 	fprintf form "@[<hov 1>(raise@ %s)@]" id
@@ -1052,32 +1137,43 @@ let rec fprintf_expr_node form e =
              fprintf form "(%s : %a) " (why3id_if x)
                (fprintf_type ~need_colon:false false) t)
 	  params;
-	fprintf form "@]->@ @[<hov 0>{ ";
-	if pre <> LTrue
-	then fprintf_assertion form pre;
-	fprintf form " }@ %a@]@ " fprintf_expr body;
-	begin
-	  match signals with
-	    | [] ->
-		fprintf form "@[<hov 2>{ %a }@]@]" fprintf_assertion post
-	     | l ->
-                 if !why3syntax then
-		   fprintf form "@[<hov 2>{ %a@ } | %a @]"
-		     fprintf_assertion post
-		     (print_list alt
-		        (fun fmt (e,r) ->
-			   fprintf fmt "@[<hov 2>%s ->@ { %a }@]" e
-			     fprintf_assertion r))
-		     l
-                 else
-		   fprintf form "@[<hov 2>{ %a@ | %a }@]"
-		     fprintf_assertion post
-		     (print_list alt
-		        (fun fmt (e,r) ->
-			   fprintf fmt "@[<hov 2>%s =>@ %a@]" e
-			     fprintf_assertion r))
-		     l
-	end;
+        if !why3syntax then
+          begin
+	    fprintf form "@]->@ @[<hov 0>requires { %a  }@ "
+              fprintf_assertion pre;
+            begin
+	    match signals with
+	      | [] ->
+		  fprintf form "@[<hov 2>ensures { %a }@]@]" fprintf_assertion post
+	      | l ->
+		    fprintf form "@[<hov 2> ensures { %a@ } %a @]"
+		      fprintf_assertion post
+		      (print_list alt
+		         (fun fmt (e,r) ->
+			    fprintf fmt "@[<hov 2>raises { %s result ->@ %a }@]" e
+			      fprintf_assertion r))
+		      l
+            end;
+	    fprintf form "%a@]@ " fprintf_expr body;
+          end
+        else
+          begin
+	    fprintf form "@]->@ @[<hov 0>{ ";
+	    if pre <> LTrue
+	    then fprintf_assertion form pre;
+	    fprintf form " }@ %a@]@ " fprintf_expr body;
+	    match signals with
+	      | [] ->
+		  fprintf form "@[<hov 2>{ %a }@]@]" fprintf_assertion post
+	      | l ->
+		  fprintf form "@[<hov 2>{ %a@ | %a }@]"
+		    fprintf_assertion post
+		    (print_list alt
+		       (fun fmt (e,r) ->
+			  fprintf fmt "@[<hov 2>%s =>@ %a@]" e
+			    fprintf_assertion r))
+		    l
+	  end;
         List.iter
 	  (fun (x,t) ->
              (match t with
@@ -1090,13 +1186,35 @@ let rec fprintf_expr_node form e =
 	  fprintf_assertion pre
 	  fprintf_expr e
     | Triple(o,pre,e,post,exceps) ->
-	fprintf form "@[<hov 0>(assert { %a };@ ((%a)@ "
-	  fprintf_assertion pre
-	  fprintf_expr e;
+      fprintf form "@[<hov 0>(assert { %a };@ " fprintf_assertion pre;
+      if !why3syntax then
+        if o then
+          begin
+            fprintf form "abstract@ ensures {@ %a }@ "
+	      fprintf_assertion post;
+            List.iter
+              (fun (e,r) ->
+                fprintf form "@[<hov 2>raises { %s ->@ %a }@]" e fprintf_assertion r)
+              exceps;
+            fprintf form "@ %a@ end"
+              fprintf_expr e
+          end
+        else
+          begin
+            match exceps with
+	      | [] ->
+                fprintf form "let _ = %a in assert { %a }"
+                  fprintf_expr e
+		  fprintf_assertion post
+              | _ -> assert false (* TODO *)
+        end
+      else
 	begin
+          fprintf form "(%a)@ " fprintf_expr e;
 	  match exceps with
 	    | [] ->
-		(if o then fprintf form "{{ %a }}" else fprintf form "{ %a }")
+		(if o then
+                    fprintf form "{{ %a }}" else fprintf form "{ %a }")
 		  fprintf_assertion post
 	    | l ->
 		(if o then
@@ -1110,7 +1228,7 @@ let rec fprintf_expr_node form e =
 		       fprintf_assertion r))
 		  l
 	end;
-	fprintf form "))@]"
+      fprintf form ")@]"
     | Assert(k,p, e) ->
 	fprintf form "@[<hov 0>(%s@ { %a };@ %a)@]"
           (match k with `ASSERT -> "assert" | `CHECK -> "check")
@@ -1131,10 +1249,10 @@ let rec fprintf_expr_node form e =
 	  (l.pos_cnum - l.pos_bol) fprintf_expr e
 	*)
 
-and fprintf_expr form e =
+and fprintf_expr_gen in_app form e =
   let rec aux l =
     match l with
-      | [] -> fprintf_expr_node form e.expr_node
+      | [] -> fprintf_expr_node in_app form e.expr_node
       | s::l ->
 (*
           if s="L2" then Format.eprintf "Output.fprintf_expr: printing label %s for expression %a@." s fprintf_expr_node e.expr_node;
@@ -1147,6 +1265,8 @@ and fprintf_expr form e =
           fprintf form ")@]"
   in aux e.expr_labels
 
+and fprintf_expr form e = fprintf_expr_gen false form e
+
 and fprintf_expr_list form l =
   match l with
     | [] -> ()
@@ -1182,18 +1302,31 @@ let make_and_expr a1 a2 =
 let make_app_rec ~logic f l =
   let rec make_rec accu = function
     | [] -> accu
-    | e::r -> make_rec (mk_expr (App(accu,e))) r
+    | e::r -> make_rec (mk_expr (App(accu,e,None))) r
   in
   match l with
     | [] -> if logic then make_rec f [] else make_rec f [void]
     | l -> make_rec f l
 ;;
 
-let make_app id l = make_app_rec ~logic:false (mk_var id) l
+let app_add_ty ?ty e =
+  match ty,e.expr_node with
+  | None,_ -> e
+  | Some _, App(e1,e2,None) -> { e with expr_node = App(e1,e2,ty)}
+  | Some _, _ ->
+    invalid_arg "Output.app_add_ty: type coercion for constant. To correct."
 
-let make_logic_app id l = make_app_rec ~logic:true (mk_var id) l
+let make_app ?ty id l =
+  let app = make_app_rec ~logic:false (mk_var id) l in
+  app_add_ty ?ty app
 
-let make_app_e = make_app_rec ~logic:false
+let make_logic_app ?ty id l =
+  let app = make_app_rec ~logic:true (mk_var id) l in
+  app_add_ty ?ty app
+
+let make_app_e ?ty e l =
+  let app = make_app_rec ~logic:false e l in
+  app_add_ty ?ty app
 
 let make_while cond inv var e =
   let body =
@@ -1206,6 +1339,10 @@ let make_label label e =
 (*
   if label = "L2" then Format.eprintf "Output.make_label: adding label %s@." label;
 *)
+  assert (String.length label > 0);
+  if label.[0] = '_' then
+  { e with expr_loc_labels = label :: e.expr_loc_labels }
+  else
   { e with expr_labels = label :: e.expr_labels }
 
 let make_pre pre e =  mk_expr (Triple(false,pre,e,LTrue,[]))
@@ -1227,7 +1364,8 @@ let append_list e l =
     | e'::rem ->
         match e.expr_node,e'.expr_node with
           | MultiAssign(mark1,pos1,lets1,isrefa1,ta1,a1,tmpe1,e1,f1,l1),
-            MultiAssign(_,_,lets2,_isrefa2,_ta2,a2,_tmpe2,e2,f2,l2) ->
+            MultiAssign(_,_,lets2,_isrefa2,_ta2,a2,_tmpe2,e2,f2,l2)
+            when e'.expr_labels = [] ->
               (*
                 Format.eprintf
                 "Found multi-assigns: a1=%a, a2=%a, e1=%a, e2=%a, f1=%s,f2=%s@."
@@ -1241,7 +1379,8 @@ let append_list e l =
                     (*
                       Format.eprintf "append_list, merge successful!@.";
                     *)
-                    { expr_labels = e.expr_labels @ e'.expr_labels ;
+                    { expr_labels = e.expr_labels;
+                      expr_loc_labels = e.expr_loc_labels @ e'.expr_loc_labels;
                       expr_node =
                         MultiAssign(mark1,pos1,lets1@lets2,isrefa1,ta1,a1,tmpe1,e1,f1,l) }
                     ::rem
@@ -1276,26 +1415,49 @@ let append_list e l =
               e::l
 
 
-let make_block labels l =
+let make_block labels loc_labels l =
   match l with
       | [] -> assert false
-      | [e] -> {e with expr_labels = labels @ e.expr_labels }
-      | _ -> { expr_labels = labels ; expr_node = Block l }
+      | [e] -> {e with expr_labels = labels @ e.expr_labels ; 
+                       expr_loc_labels = loc_labels @ e.expr_loc_labels }
+      | _ -> { expr_labels = labels ; expr_loc_labels = loc_labels ; expr_node = Block l }
 
+(** Try hard to keep the labels visible by all the instructions that follow
+    but also to remove unneeded block
+
+    The principle is to push e2 inside e1 such that every labels visible for
+    the last instruction (not a block) of e1 can be visible for e2
+ *)
+let rec append' e1 e2 =
+  match e1.expr_node,e2.expr_node with
+    | Void,_ -> (* assert (e1.expr_labels = []);*) [e2]
+    | _,Void -> assert (e2.expr_labels = []); [e1]
+    | Block(_),Block([]) -> [e1]
+    | Block(l1),_ ->
+      [make_block e1.expr_labels e1.expr_loc_labels (concat l1 e2)]
+    | _,Block(l2) ->
+      begin match e1.expr_labels, e2.expr_labels with
+        | [], [] -> append_list e1 l2
+        | labels1, [] ->
+          [make_block labels1 e1.expr_loc_labels (append_list {e1 with expr_labels = []} l2)]
+        | labels1, _ ->
+          [make_block labels1 e1.expr_loc_labels (append_list {e1 with expr_labels = []} [e2])]
+      end
+    | _ ->
+      if e1.expr_labels = [] then
+        append_list e1 [e2]
+      else
+        let e1' = {e1 with expr_labels = []} in
+        [make_block e1.expr_labels [] (append_list e1' [e2])]
+
+and concat l1 e2 =
+  match l1 with
+  | [] -> [e2]
+  | [a1] -> append' a1 e2
+  | a1::l1 -> a1::(concat l1 e2)
 
 let append e1 e2 =
-  match e1.expr_node,e2.expr_node with
-    | Void,_ -> (* assert (e1.expr_labels = []);*) e2
-    | _,Void -> assert (e2.expr_labels = []); e1
-    | Block(l1),Block(l2) ->
-        make_block (e1.expr_labels@e2.expr_labels)
-          (List.fold_right append_list l1 l2)
-    | Block(l1),_ ->
-        make_block e1.expr_labels (List.fold_right append_list l1 [e2])
-    | _,Block(l2) ->
-        make_block e2 .expr_labels (append_list e1 l2)
-    | _ -> make_block [] (append_list e1 [e2])
-
+  make_block [] [] (append' e1 e2)
 
 type goal_kind = KAxiom | KLemma | KGoal
 
@@ -1305,7 +1467,7 @@ let id_no_loc s = { name = s; loc = Loc.dummy_floc }
 
 type why_decl =
   | Param of bool * why_id * why_type         (*r parameter in why *)
-  | Def of why_id * expr               (*r global let in why *)
+  | Def of why_id * bool * expr               (*r global let in why *)
   | Logic of bool * why_id * (string * logic_type) list * logic_type    (*r logic decl in why *)
   | Predicate of bool * why_id * (string * logic_type) list * assertion
   | Inductive of bool * why_id * (string * logic_type) list *
@@ -1321,7 +1483,7 @@ let get_why_id d =
   match d with
     | Param(_,id,_)
     | Logic(_,id,_,_)
-    | Def(id,_)
+    | Def(id,_,_)
     | Goal(_,id,_)
     | Predicate(_,id,_,_)
     | Function(_,id,_,_,_)
@@ -1332,7 +1494,7 @@ let get_why_id d =
 let iter_why_decl f d =
   match d with
     | Param(_,_,t) -> iter_why_type f t
-    | Def(_id,t) -> iter_expr f t
+    | Def(_id,_,t) -> iter_expr f t
     | Logic(_,_id,args,t) ->
 	List.iter (fun (_,t) -> iter_logic_type f t) args;
 	iter_logic_type f t
@@ -1424,7 +1586,7 @@ let fprintf_why_decl form d =
 	(if b then "external " else "") (why3ident id.name)
 	  (fprintf_type ~need_colon:true false) t
     | Param(b,id,t) ->
-	fprintf form "@[<hov 1>%sparameter %s :@ %a@]@.@."
+	fprintf form "@[<hov 1>%sparameter %s :@ %a@]@\n@."
 	(if b then "external " else "") id.name
 	  (fprintf_type ~need_colon:false false) t
     | Logic(b,id,args,t) when !why3syntax ->
@@ -1470,12 +1632,13 @@ let fprintf_why_decl form d =
 	fprintf form "@[<hov 1>%s %s :@ %a@]@.@." (str_of_goal_kind k)
           id.name
 	  fprintf_assertion p
-    | Def(id,e) when !why3syntax ->
-        fprintf form "@[<hov 1>let %s %a=@ %a@]@.@."
+    | Def(id,diverges,e) when !why3syntax ->
+        fprintf form "@[<hov 1>let %s%s %a=@ %a@]@.@."
           (why3id id.name)
+          (if diverges then " \"W:diverges:N\"" else "")
           (why3loc ~prog:false) id.name
           fprintf_expr e
-    | Def(id,e) ->
+    | Def(id,_,e) ->
         fprintf form "@[<hov 1>let %s =@ %a@]@.@." (why3id_if id.name)
           fprintf_expr e
     | Predicate (b, id, args, p) when !why3syntax ->
@@ -1531,48 +1694,57 @@ let output_decls get_id iter_decl output_decl decls =
        do_topo map iter_decl output_decl id.name decl)
     l
 
-let output_why3_imports form use_floats full_floats =
+let output_why3_imports form use_floats float_model =
   fprintf form "use import int.Int@\n@\n";
-  fprintf form "use import bool.Bool@\n@\n";
+  fprintf form "use bool.Bool@\n@\n";
   if !why3_IntMinMax then
-    fprintf form "use import int.MinMax as IntMinMax@\n@\n";
+    fprintf form "use int.MinMax as IntMinMax@\n@\n";
   if !why3_ComputerDivision then
-    fprintf form "use import int.ComputerDivision@\n@\n";
+    fprintf form "use int.ComputerDivision@\n@\n";
   if !why3_reals then
     fprintf form "use import real.RealInfix@\n@\n";
   if !why3_FromInt then
-    fprintf form "use import real.FromInt@\n@\n";
+    fprintf form "use real.FromInt@\n@\n";
   if !why3_Truncate then
-    fprintf form "use import real.Truncate@\n@\n";
+    fprintf form "use real.Truncate@\n@\n";
   if !why3_Square then
-    fprintf form "use import real.Square@\n@\n";
+    fprintf form "use real.Square@\n@\n";
   if !why3_Power then
-    fprintf form "use import real.Power@\n@\n";
+    fprintf form "use int.Power@\n@\n";
+  if !why3_PowerInt then
+    fprintf form "use real.PowerInt@\n@\n";
+  if !why3_PowerReal then
+    fprintf form "use real.PowerReal@\n@\n";
   if !why3_RealMinMax then
-    fprintf form "use import real.MinMax as RealMinMax@\n@\n";
+    fprintf form "use real.MinMax as RealMinMax@\n@\n";
+  if !why3_AbsInt then
+    fprintf form "use int.Abs as AbsInt@\n@\n";
   if !why3_AbsReal then
-    fprintf form "use import real.Abs as AbsReal@\n@\n";
+    fprintf form "use real.Abs as AbsReal@\n@\n";
+  if !why3_ExpLog then
+    fprintf form "use real.ExpLog@\n@\n";
   if !why3_Trigonometry then
-    fprintf form "use import real.Trigonometry@\n@\n";
+    fprintf form "use real.Trigonometry@\n@\n";
   if use_floats then
     begin
-      if full_floats then
-        begin
-          fprintf form "use import floating_point.SingleFull@\n";
-          fprintf form "use import floating_point.DoubleFull@\n@\n";
-        end
-      else
-        begin
+      match float_model with
+        | Jc_env.FMfull ->
+          fprintf form "use import floating_point.SingleFull as Single@\n";
+          fprintf form "use import floating_point.DoubleFull as Double@\n@\n"
+        | Jc_env.FMdefensive ->
           fprintf form "use import floating_point.Single@\n";
-          fprintf form "use import floating_point.Double@\n@\n";
-        end
+          fprintf form "use import floating_point.Double@\n@\n"
+        | Jc_env.FMmultirounding ->
+          (* fprintf form "use import floating_point.Single@\n"; *)
+          fprintf form "use import floating_point.DoubleMultiRounding as Double@\n@\n"
+        | Jc_env.FMmath ->
+          assert false (* TODO *)
     end;
-  fprintf form "use import jessie3.Jessie_memory_model@\n@\n"
+  fprintf form "use import jessie3theories.Jessie_memory_model@\n@\n"
 
 
 let fprintf_why_decls ?(why3=false) ?(use_floats=false)
-    ?(full_floats=false)
-    form decls =
+    ~float_model form decls =
   why3syntax := why3;
   (* Why do we need a partition ?
      because one may have a type and a logic/parameter with the same name,
@@ -1629,27 +1801,32 @@ let fprintf_why_decls ?(why3=false) ?(use_floats=false)
   if why3 then
     begin
       fprintf form "theory Jessie_model@\n@\n";
-      output_why3_imports form use_floats full_floats
+      output_why3_imports form use_floats float_model
     end;
   output_decls get_why_id iter_why_decl (fprintf_why_decl form) types;
   output_decls get_why_id iter_why_decl (fprintf_why_decl form) others;
-  if why3 then 
+  if why3 then
     begin
       fprintf form "end@\n@\n";
       fprintf form "module Jessie_program@\n@\n";
-      output_why3_imports form use_floats full_floats;
+      output_why3_imports form use_floats float_model;
       fprintf form "use import Jessie_model@\n@\n";
-      fprintf form "use import module ref.Ref@\n@\n";
-      fprintf form "use import module jessie3.JessieDivision@\n@\n";
+      fprintf form "use import ref.Ref@\n@\n";
+      fprintf form "use import jessie3.JessieDivision@\n@\n";
       if use_floats then
         begin
           fprintf form "use import floating_point.Rounding@\n@\n";
-          if full_floats then
-            fprintf form "use import module jessie3.JessieFloatsFull@\n@\n"
-          else
-            fprintf form "use import module jessie3.JessieFloats@\n@\n";
+          match float_model with
+            | Jc_env.FMfull ->
+              fprintf form "use import jessie3.JessieFloatsFull@\n@\n"
+            | Jc_env.FMdefensive ->
+              fprintf form "use import jessie3.JessieFloats@\n@\n"
+            | Jc_env.FMmultirounding ->
+              fprintf form "use import jessie3.JessieFloatsMultiRounding@\n@\n"
+            | Jc_env.FMmath -> assert false (* TODO *)
         end;
-      fprintf form "use import module jessie3.Jessie_memory_model_parameters@\n@\n";
+      fprintf form "use import jessie3.Jessie_memory_model_parameters@\n@\n";
+      fprintf form "use import jessie3_integer.Integer@\n@\n";
     end;
   output_decls get_why_id iter_why_decl (fprintf_why_decl form) params;
   output_decls get_why_id iter_why_decl (fprintf_why_decl form) defs;
