@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -156,7 +156,7 @@ type expr_node =
       *)
   | Let of string * expr * expr
   | Let_ref of string * expr * expr
-  | App of expr * expr
+  | App of expr * expr * why_type option
   | Raise of string * expr option
   | Try of expr * string * string option * expr
   | Fun of (string * why_type) list *
@@ -171,8 +171,9 @@ type expr_node =
   | Absurd
   | Loc of Lexing.position * expr
 
-and expr =
+and expr = 
     { expr_labels : string list;
+      expr_loc_labels : string list;
       expr_node : expr_node;
     }
 
@@ -182,7 +183,9 @@ val mk_expr : expr_node -> expr
 val mk_var : string -> expr
 val void : expr
 
-val make_block : string list -> expr list -> expr
+(*
+val make_block : string list -> string list -> expr list -> expr
+*)
 
 val fprintf_expr : Format.formatter -> expr -> unit
 
@@ -197,15 +200,16 @@ val make_and_expr : expr -> expr -> expr
 
 *)
 
-val make_app : string -> expr list -> expr
-val make_logic_app : string -> expr list -> expr
-val make_app_e : expr -> expr list -> expr
+val make_app : ?ty:why_type -> string -> expr list -> expr
+val make_logic_app : ?ty:why_type -> string -> expr list -> expr
+val make_app_e : ?ty:why_type -> expr -> expr list -> expr
 
 (*
 
   [make_label l e] adds label l to e
 
-
+  In the case of a block, add it to the first instruction of the block, if there
+  is one.
 *)
 
 val make_label : string -> expr -> expr;;
@@ -233,7 +237,8 @@ type goal_kind = KAxiom | KLemma | KGoal
 
 type why_decl =
   | Param of bool * why_id * why_type         (*r parameter in why *)
-  | Def of why_id * expr               (*r global let in why *)
+  | Def of why_id * bool *     (* "diverges" flag *)
+            expr               (*r global let in why *)
   | Logic of bool * why_id * (string * logic_type) list * logic_type    (*r logic decl in why *)
   | Predicate of bool * why_id * (string * logic_type) list * assertion
   | Inductive of bool * why_id * (string * logic_type) list *
@@ -246,7 +251,7 @@ type why_decl =
 val fprintf_why_decl : Format.formatter -> why_decl -> unit;;
 
 val fprintf_why_decls : ?why3:bool -> ?use_floats:bool -> 
-  ?full_floats:bool -> Format.formatter -> why_decl list -> unit
+  float_model:Jc_env.float_model -> Format.formatter -> why_decl list -> unit
 
 type kind =
   | VarDecr
