@@ -44,11 +44,12 @@ public final class Integer extends Number implements Comparable {
      *
      * @since   JDK1.1
      */
-    public static final Class	TYPE = Class.getPrimitiveClass("int");
+    //KML public static final Class	TYPE = Class.getPrimitiveClass("int");
 
     /**
      * All possible chars for representing a number as a String
      */
+    /* KML
     final static char[] digits = {
 	'0' , '1' , '2' , '3' , '4' , '5' ,
 	'6' , '7' , '8' , '9' , 'a' , 'b' ,
@@ -57,6 +58,7 @@ public final class Integer extends Number implements Comparable {
 	'o' , 'p' , 'q' , 'r' , 's' , 't' ,
 	'u' , 'v' , 'w' , 'x' , 'y' , 'z'
     };
+    */
 
     /**
      * Returns a string representation of the first argument in the
@@ -236,7 +238,7 @@ public final class Integer extends Number implements Comparable {
 	return new String(buf, charPos, (32 - charPos));
     }
 
-
+    /* KML
     final static char [] DigitTens = {
 	'0', '0', '0', '0', '0', '0', '0', '0', '0', '0',
 	'1', '1', '1', '1', '1', '1', '1', '1', '1', '1',
@@ -262,7 +264,7 @@ public final class Integer extends Number implements Comparable {
 	'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
 	'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
 	} ;
-
+    */
 	// I use the "invariant division by multiplication" trick to
 	// accelerate Integer.toString.  In particular we want to
 	// avoid division by 10.
@@ -436,6 +438,7 @@ public final class Integer extends Number implements Comparable {
             throw new NumberFormatException("null");
         }
 
+	/*KML
 	if (radix < Character.MIN_RADIX) {
 	    throw new NumberFormatException("radix " + radix +
 					    " less than Character.MIN_RADIX");
@@ -445,6 +448,7 @@ public final class Integer extends Number implements Comparable {
 	    throw new NumberFormatException("radix " + radix +
 					    " greater than Character.MAX_RADIX");
 	}
+	*/
 
 	int result = 0;
 	boolean negative = false;
@@ -588,6 +592,8 @@ public final class Integer extends Number implements Comparable {
      * @param   value   the value to be represented by the 
      *			<code>Integer</code> object.
      */
+    /*@ ensures this.value == value;
+      @*/
     public Integer(int value) {
 	this.value = value;
     }
@@ -629,6 +635,8 @@ public final class Integer extends Number implements Comparable {
      * Returns the value of this <code>Integer</code> as an
      * <code>int</code>.
      */
+    /*@ ensures \result == this.value;
+      @*/
     public int intValue() {
 	return value;
     }
@@ -809,7 +817,8 @@ public final class Integer extends Number implements Comparable {
      * @see java.lang.Integer#decode
      */
     public static Integer getInteger(String nm, Integer val) {
-	String v = null;
+	/*KML
+	  String v = null;
         try {
             v = System.getProperty(nm);
         } catch (IllegalArgumentException e) {
@@ -821,6 +830,7 @@ public final class Integer extends Number implements Comparable {
 	    } catch (NumberFormatException e) {
 	    }
 	}
+	*/
 	return val;
     }
 
