@@ -4,6 +4,6 @@ package com.sun.javacard.impl;
 
 public interface Constants {
 
-  public static final int APDU_BUFFER_LENGTH;
+  public static final short APDU_BUFFER_LENGTH;
 
 }
\ No newline at end of file
