@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -85,7 +85,7 @@ module X = struct
 
   let rec string_of_stmt = function
     | Void -> "void"
-    | Mix { node = PSassert p } -> "assert ... "
+    | Mix { node = PSassert _p } -> "assert ... "
     | Mix _ -> "<mix>"
     | Assume s -> "assume " ^ s
     | Seq (s1, s2) -> string_of_stmt s1 ^ "; " ^ string_of_stmt s2
@@ -174,7 +174,7 @@ let nneg s = s ^ " >= 0"
 
 (* prev = previous instruction
    lab = label of current instruction *)
-let interp_stmt self prev lab s = match s.node with
+let interp_stmt self _prev _lab s = match s.node with
   | PSinvariant i -> 
       Ainvariant i
   | PSassert a -> 
@@ -210,7 +210,7 @@ let declare_equ loc id a =
   if Hashtbl.mem equ id then error loc (ClashEqu id);
   Hashtbl.add equ id a
 
-let mixal (pseudo,asm) init =
+let mixal (pseudo,asm) _init =
   let self = ref 0 in
   (* pseudo *)
   List.iter
