@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -47,72 +47,6 @@ let push_decl d =
 
 let iter = Encoding.iter
 
-let is_why3_keyword =
-  let ht = Hashtbl.create 43 in
-  List.iter (fun kw -> Hashtbl.add ht kw ())
-    [
-        "as";
-        "axiom";
-        "clone";
-        "else";
-        "end";
-        "epsilon";
-        "exists";
-        "export";
-        "false";
-        "forall";
-        "function";
-        "goal";
-        "if";
-        "import";
-        "in";
-        "inductive";
-        "lemma";
-        "let";
-        "match";
-        "meta";
-        "namespace";
-        "not";
-        "predicate";
-        "prop";
-        "then";
-        "theory";
-        "true";
-        "type";
-        "use";
-        "with";
-        "abstract";
-        "absurd";
-        "any";
-        "assert";
-        "assume";
-        "begin";
-        "check";
-        "do";
-        "done";
-        "downto";
-        "exception";
-        "for";
-        "fun";
-        "invariant";
-        "label";
-        "loop";
-        "model";
-        "module";
-        "mutable";
-        "raise";
-        "raises";
-        "reads";
-        "rec";
-        "to";
-        "try";
-        "val";
-        "variant";
-        "while";
-        "writes";
-    ];
-  Hashtbl.mem ht
-
 
 (*
   let capitalize s =
@@ -124,12 +58,12 @@ let is_why3_keyword =
 let ident fmt s =
   let s = Ident.string s in
   let s =
-    if is_why3_keyword s then s ^ "_why3" else s
+    if Why3_kw.is_why3_keyword s then s ^ "_why3" else s
   in
   let s = 
     if 'A' <= s.[0] && s.[0] <= 'Z' then "_" ^ s  else s
   in
-  if is_why3_keyword s then
+  if Why3_kw.is_why3_keyword s then
     fprintf fmt "why3__%s" s
   else
     pp_print_string fmt s
