@@ -2,21 +2,21 @@
 /*                                                                        */
 /*  The Why platform for program certification                            */
 /*                                                                        */
-/*  Copyright (C) 2002-2011                                               */
+/*  Copyright (C) 2002-2014                                               */
 /*                                                                        */
-/*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                */
-/*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           */
-/*    Yannick MOY, Univ. Paris-sud 11                                     */
-/*    Romain BARDOU, Univ. Paris-sud 11                                   */
+/*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   */
+/*    Claude MARCHE, INRIA & Univ. Paris-sud                              */
+/*    Yannick MOY, Univ. Paris-sud                                        */
+/*    Romain BARDOU, Univ. Paris-sud                                      */
 /*                                                                        */
 /*  Secondary contributors:                                               */
 /*                                                                        */
-/*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     */
-/*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          */
-/*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        */
-/*    Sylvie BOLDO, INRIA                 (floating-point support)        */
-/*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  */
-/*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       */
+/*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        */
+/*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             */
+/*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           */
+/*    Sylvie BOLDO, INRIA              (floating-point support)           */
+/*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     */
+/*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          */
 /*                                                                        */
 /*  This software is free software; you can redistribute it and/or        */
 /*  modify it under the terms of the GNU Lesser General Public            */
@@ -82,7 +82,7 @@
   @*/
 
 /*@ lemma gcd_property :
-  @   \forall integer a b d q;
+  @   \forall integer a b d;
   @      a >= 0 && b > 0 && isGcd(b,a % b,d) ==> isGcd(a,b,d) ;
   @*/
 
@@ -125,7 +125,7 @@ class Gcd {
 
 /*
 Local Variables:
-compile-command: "make Gcd.why3ml"
+compile-command: "make Gcd.why3ide"
 End:
 */
 
