@@ -2,21 +2,21 @@
 /*                                                                        */
 /*  The Why platform for program certification                            */
 /*                                                                        */
-/*  Copyright (C) 2002-2011                                               */
+/*  Copyright (C) 2002-2014                                               */
 /*                                                                        */
-/*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                */
-/*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           */
-/*    Yannick MOY, Univ. Paris-sud 11                                     */
-/*    Romain BARDOU, Univ. Paris-sud 11                                   */
+/*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   */
+/*    Claude MARCHE, INRIA & Univ. Paris-sud                              */
+/*    Yannick MOY, Univ. Paris-sud                                        */
+/*    Romain BARDOU, Univ. Paris-sud                                      */
 /*                                                                        */
 /*  Secondary contributors:                                               */
 /*                                                                        */
-/*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     */
-/*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          */
-/*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        */
-/*    Sylvie BOLDO, INRIA                 (floating-point support)        */
-/*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  */
-/*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       */
+/*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        */
+/*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             */
+/*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           */
+/*    Sylvie BOLDO, INRIA              (floating-point support)           */
+/*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     */
+/*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          */
 /*                                                                        */
 /*  This software is free software; you can redistribute it and/or        */
 /*  modify it under the terms of the GNU Lesser General Public            */
@@ -37,11 +37,11 @@
   @   \forall integer i j, int t[];
   @    i >= j ==> num_of_pos(i,j,t) == 0;
   @  axiom num_of_pos_true_case{L} :
-  @   \forall integer i j k, int t[];
+  @   \forall integer i j, int t[];
   @       i < j && t[j-1] > 0 ==>
   @         num_of_pos(i,j,t) == num_of_pos(i,j-1,t) + 1;
   @  axiom num_of_pos_false_case{L} :
-  @   \forall integer i j k, int t[];
+  @   \forall integer i j, int t[];
   @       i < j && ! (t[j-1] > 0) ==>
   @         num_of_pos(i,j,t) == num_of_pos(i,j-1,t);
   @ }
@@ -102,6 +102,6 @@ public class Muller {
 
 /*
 Local Variables:
-compile-command: "make Muller.why3ml"
+compile-command: "make Muller.why3ide"
 End:
 */
