@@ -2,21 +2,21 @@
 (*                                                                        *)
 (*  The Why platform for program certification                            *)
 (*                                                                        *)
-(*  Copyright (C) 2002-2011                                               *)
+(*  Copyright (C) 2002-2014                                               *)
 (*                                                                        *)
-(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud 11                *)
-(*    Claude MARCHE, INRIA & Univ. Paris-sud 11                           *)
-(*    Yannick MOY, Univ. Paris-sud 11                                     *)
-(*    Romain BARDOU, Univ. Paris-sud 11                                   *)
+(*    Jean-Christophe FILLIATRE, CNRS & Univ. Paris-sud                   *)
+(*    Claude MARCHE, INRIA & Univ. Paris-sud                              *)
+(*    Yannick MOY, Univ. Paris-sud                                        *)
+(*    Romain BARDOU, Univ. Paris-sud                                      *)
 (*                                                                        *)
 (*  Secondary contributors:                                               *)
 (*                                                                        *)
-(*    Thierry HUBERT, Univ. Paris-sud 11  (former Caduceus front-end)     *)
-(*    Nicolas ROUSSET, Univ. Paris-sud 11 (on Jessie & Krakatoa)          *)
-(*    Ali AYAD, CNRS & CEA Saclay         (floating-point support)        *)
-(*    Sylvie BOLDO, INRIA                 (floating-point support)        *)
-(*    Jean-Francois COUCHOT, INRIA        (sort encodings, hyps pruning)  *)
-(*    Mehdi DOGGUY, Univ. Paris-sud 11    (Why GUI)                       *)
+(*    Thierry HUBERT, Univ. Paris-sud  (former Caduceus front-end)        *)
+(*    Nicolas ROUSSET, Univ. Paris-sud (on Jessie & Krakatoa)             *)
+(*    Ali AYAD, CNRS & CEA Saclay      (floating-point support)           *)
+(*    Sylvie BOLDO, INRIA              (floating-point support)           *)
+(*    Jean-Francois COUCHOT, INRIA     (sort encodings, hyps pruning)     *)
+(*    Mehdi DOGGUY, Univ. Paris-sud    (Why GUI)                          *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
 (*  modify it under the terms of the GNU Lesser General Public            *)
@@ -45,29 +45,30 @@ type prover_result =
 
 val cpulimit : string ref
 
-type prover = ?debug:bool -> ?timeout:int -> ?filename:string -> ?buffers:(Buffer.t list) -> unit ->
+type prover = ?debug:bool -> ?switch:string -> 
+    ?timeout:int -> ?filename:string -> ?buffers:(Buffer.t list) -> unit ->
   prover_result
 
-val gappa : ?debug:bool -> ?timeout:int -> filename:string -> unit ->
-  prover_result
+val gappa : ?debug:bool ->  ?switch:string ->
+  ?timeout:int -> filename:string -> unit -> prover_result
 
-val simplify : ?debug:bool -> ?timeout:int -> filename:string -> unit ->
-  prover_result
+val simplify : ?debug:bool ->  ?switch:string -> 
+  ?timeout:int -> filename:string -> unit ->prover_result
 
-val vampire : ?debug:bool -> ?timeout:int -> filename:string -> unit ->
-  prover_result
+val vampire : ?debug:bool ->  ?switch:string -> 
+  ?timeout:int -> filename:string -> unit -> prover_result
 
 val harvey :
-  ?debug:bool -> ?timeout:int -> filename:string -> unit ->
-  prover_result
+  ?debug:bool ->  ?switch:string -> ?timeout:int -> 
+  filename:string -> unit -> prover_result
 
 val zenon :
-  ?debug:bool -> ?timeout:int -> filename:string -> unit ->
-  prover_result
+  ?debug:bool ->  ?switch:string -> ?timeout:int -> 
+  filename:string -> unit -> prover_result
 
 val rvsat :
-  ?debug:bool -> ?timeout:int -> filename:string -> unit ->
-  prover_result
+  ?debug:bool ->  ?switch:string -> ?timeout:int -> 
+  filename:string -> unit -> prover_result
 
 val yices : prover
 
@@ -82,13 +83,13 @@ val verit: prover
 val ergo : select_hypotheses:bool -> prover
 
 val generic_hypotheses_selection :
-  ?debug:bool ->
+  ?debug:bool -> ?switch:string ->
   ?timeout:int -> filename:string -> DpConfig.prover_id -> unit -> prover_result
 
 val coq :
-  ?debug:bool ->
+  ?debug:bool -> ?switch:string ->
   ?timeout:int -> filename:string -> unit -> prover_result
 
 val pvs :
-  ?debug:bool ->
+  ?debug:bool -> ?switch:string ->
   ?timeout:int -> filename:string -> unit -> prover_result
